###################################
# (C) Copyright International Business Machines (IBM), 2001, 2002, 2005
# Useable and distributable only with IBM permission and under IBM terms and
# conditions. It may not be copied or further distributed unless authorized in
# writing by IBM.
###################################
# use strict;
# use diagnostics -verbose;
use port_structs;

use integer;
use Math::BigFloat;

#SCOPE VARIABLES
#counters and flags
our $master_flag  = 0;    #set to 1 on any error. this is what is returned.
our $find_flag    = 0;
our $open_flag    = 0;
our $doc_flag     = 0;
our $remove_flag  = 0;
our $portout_flag = 0;
our $include_file_marked_flag = 0;
our $easy_count               = 0;
our $mod_count                = 0;
our $diff_count               = 0;
our $flagged_line_count       = 0;
our $total_line_count         = 0;
our $unknown_count            = 0;
our $assembler_count          = 0;
our $fcn_count                = 0;
our $loop_count;
our $line_count;
our $cur_fcn;
our $cur_line;
our $counter;
our $fcount;

#arrays
our @sort_file_out;
our @fcn_element;
our @lines;
our @FILES;
our @a_fields;
our @b_fields;
our @portouts;
our @file_out;    #an array of Problems cumulative for the whole project
                  #gets output in format to port_catagories.out file

#hashes
our %incls;
our %fcs;
our %pragma;
our %fun_call;
our %param_check;

#used for math
our $flag_percent;
our $diff_percent;
our $mod_percent;
our $easy_percent;
our $unknown_percent;

#misc variables
our $cur_func_doc;
our $source_file;
our $line_index;
our $tstring;
our $incl_line;
our $funpat;
our $curr_file;
our $temp;
our $key;
our $pattern;
our $fn_name;
our $params;
our $diff_level;
our $file_perm;
our $perm_shift;
our $chmod;
our $curr_dir;

#our $message_parm;
our $message_row;
our $lead;
our $flagged_item;

#########################################################################
# scanner($$$\$$) - the main backend routine
#
# inputs: directory name to scan
#         include flag - whether or not to include doc in portout files
#
# output: message parameter - passed in by referece, output needed by
#                             front end component to display messages
########################################################################
sub scanner($$$\$$) {
    my (
        $parm_curr_dir, $parm_includedoc, $parm_moduledir,
        $message_parm,  $updateProgress
      )
      = @_;    # get the parameters passed in
    $fcn_count          = 0;
    $curr_dir    = $parm_curr_dir;    # copy current directory to local variable
    $message_row = 0;                 # initialize to zero

#need to reset the flags within the scope of the scanner() in case they do 2+ scans.
    $master_flag        = 0;
    $find_flag          = 0;
    $open_flag          = 0;
    $doc_flag           = 0;
    $remove_flag        = 0;
    $portout_flag       = 0;
    $easy_count         = 0;
    $mod_count          = 0;
    $diff_count         = 0;
    $flagged_line_count = 0;
    $flagged_line_count = 0;
    $total_line_count   = 0;
    $diff_count         = 0;
    $mod_count          = 0;
    $easy_count         = 0;
    $unknown_count      = 0;
    $assembler_count    = 0;

    $curr_dir =~ s/\/+/\//
      ; #We sometimes get to many /'s. Clean it up here so it prints nice in the Metrics file
    @file_out = ();    #clean out the array.

#populate hases with data
#if there is an error in these calls, we consider the error fatal to the scanner subroutine. return the flag and exit out.
    if ( !read_funcs( $parm_moduledir, $message_parm ) ) {
        return $master_flag;
    }
    if ( !read_incls( $parm_moduledir, $message_parm ) ) {
        return $master_flag;
    }
    if ( !read_pragmas( $parm_moduledir, $message_parm ) ) {
        return $master_flag;
    }

#need to read in the details for certain functions that need to have their parameters scanned.
    if ( !read_parameters( $parm_moduledir, $message_parm ) ) {
        return $master_flag;
    }

    #remove any existing files created from an earlier run
    remove_portouts($message_parm);


##################################
    #code to find matching parenthesis
##################################
#The reg ex is grouped in a manner to return three variables, $1, $2, $3
#$1 will contain the entire function call. It will contain the name of the function being called, as well as the parameters being passed
#$2 will contain the name of the function being called.
#$3 will contain the parameters being passed in the function call in question.
#
#an example is given:
#
# the following line of code (non-working) would be broken down as such
#    int (malloc(NULL, 0, malloc(NULL), NULL, malloc(NULL, NULL, NULL), NULL))
#
#$1 = malloc(NULL, 0, malloc(NULL), NULL, malloc(NULL, NULL, NULL), NULL)
#$2 = malloc
#$3 = (NULL, 0, malloc(NULL), NULL, malloc(NULL, NULL, NULL), NULL)
#
###################################
    my $np3 = qr{
  \(
    (?:
     (?>[^()]+ )
    )*
      \)
    }x;

    my $np2 = qr{
  \(
    (?:
     (?>[^()]+ )
     |
     (??{ $np3 })
    )*
      \)
    }x;

    my $np1 = qr{
  \(
    (?:
     (?>[^()]+ )
     |
     (??{ $np2 })
    )*
      \)
    }x;

    my $np = qr{
  \(
    (?:
     (?>[^()]+ )
     |
     (??{ $np1 })
    )*
      \)
    }x;

    $funpat = qr/((\W?)(\w+)\s*($np))/;
####################################
####################################

    #open FILES and read into an array
    open FILES,
"find $curr_dir -name '*.[chlsyCHLS]' -o -name '*.cc' -o -name '*.cpp' -o -name '*.hpp' -o -name '*.cxx' -o -name '*.hxx' -o -name '*.hh' 2>&1 |";
    @FILES = <FILES>;
    close FILES;

    my %filenames;
    foreach (@FILES) {
        $_ =~ m/$curr_dir\/.*?(\w*\.\w*)\n/;
        $filenames{$1} = 1;
    }

    my $file_count = scalar(@FILES);

    if ( $file_count == 0 )
    {    #not good at all.we can't find any source code files to scan.
        $master_flag = 1;
        $$message_parm->{msg_parm_num_messages} =
          ( $$message_parm->{msg_parm_num_messages} ) +
          1;    # increment number of messages
                # set up insert1
        $$message_parm->{msg_info_array_insert1}[$message_row] = $curr_dir;
        $$message_parm->{msg_info_array_hashkey}[$message_row] = "NOSOURCE";
        $message_row++;    # increment
        return $master_flag;
    }

  # Rearranged the Code : Since the SCT_files are created much before the curr_dir is scanned for source files.-Manoj
    if ( !( -e "$curr_dir/SCT_files" ) ) {
        if ( !( mkdir "$curr_dir/SCT_files" ) ) {
            $master_flag = 1;
            &log(
     "Could not create the SCT_files directory in $curr_dir. Check permissions on the folder $curr_dir. If
        they are correct, this indicates a possible corruption of the disk itself."
            );
            $$message_parm->{msg_parm_num_messages} =
              ( $$message_parm->{msg_parm_num_messages} ) +
              1;    # increment num_messages
                    # set up insert1
            $$message_parm->{msg_info_array_insert1}[$message_row] = $curr_dir;
            $$message_parm->{msg_info_array_hashkey}[$message_row] =
              "ROOTSCANDIRPROB";
            $message_row++;    # increment
            return $master_flag;
        }
    }

 #End of  Code change - Manoj

    #duplicate code
    #check and see if our output directory exists. if not, create it
    #if( !(-e "$curr_dir/SCT_files")) { mkdir "$curr_dir/SCT_files"; }

    my $current_file_nr = 0;

    foreach (@FILES) {
        ++$current_file_nr;
        %fun_call = ();
        $tstring  = '';
        chomp( $source_file = $_ );
        if ( $updateProgress->( $source_file, $current_file_nr, $file_count ) )
        {
            return 1;
        }
        if ( $source_file =~ /^find: / ) {

#there was an error searching the directory structure. we couldn't open all the dirs to search
            if ( $find_flag != 1 ) {

#only want to mark the error once. and display the error once if we haven't done that already, do so here.
                $find_flag                              = 1;
                $master_flag                            = 0;    #not fatal error
                $$message_parm->{msg_parm_num_messages} =
                  ( $$message_parm->{msg_parm_num_messages} ) +
                  1;    # increment num_messages
                        # set up insert1
                $$message_parm->{msg_info_array_insert1}[$message_row] =
                  $curr_dir;
                $$message_parm->{msg_info_array_hashkey}[$message_row] =
                  "SOMESCANFAILED";
                $message_row++;    # increment
            }

            #only show the error once, but log all errors
            &log(
"Could not scan all folders under $curr_dir. The following error was encountered\n$source_file"
            );
            next;                  #move to the next element in the array
        }

        if ( $source_file =~ /.[sS]$/ ) {

            # If its an assembler source file, count it for reporting in the
            # metrics file and skip to next file.
            $assembler_count++;
            next;
        }

        $file_perm =
          ( ( stat($source_file) )[2] ) &
          00777;    #find the current file permissions

        if ( !open( INPUT, $source_file ) ) {

            #we can't open the file.
            if ( $open_flag != 1 ) {

                # Only want to mark the error once. and display the error once.
                # If we haven't done that already, do so here.
                $open_flag                              = 1;
                $master_flag                            = 0;
                $$message_parm->{msg_parm_num_messages} =
                  ( $$message_parm->{msg_parm_num_messages} ) + 1;

                # set up insert1
                $$message_parm->{msg_info_array_insert1}[$message_row] =
                  $curr_dir;
                $$message_parm->{msg_info_array_hashkey}[$message_row] =
                  "SOMEOPENFAILED";
                $message_row++;    # increment
            }

            #only show the error once, but log all errors
            &log(
"Could not open all files under $curr_dir. The file $source_file could not be opened."
            );
            next;
        }
        else {

            #build file into a single string
            while ( $temp = <INPUT> ) {

                #parse line to check for #includes
                if ( $temp =~ m/^(\s*\#\s*include\s*<\s*)(\S+)(\s*>.*)/ ) {
                    get_include(%filenames);
                }

                #parse line to check for #pragmas
                elsif ( $temp =~ m/^(\s*\#\s*pragma\s+)(\w+)(.*)/ ) {
                    get_pragmas( $parm_moduledir, $parm_includedoc,
                        $message_parm );
                }
                $temp =~ s/\r//;    # remove DOS/WINDOWS CR line break
                $tstring .= $temp;  #build file into a single string
            }

            close INPUT;
        }    #end if-else

       # An incrementing value assigned to each flagged function found in the
       # recursive search for matching parenthesis.  The keys are sorted on this
       # value when the code is searched and the functions are replaced with the
       # function name surrounded by the appropriate markers.  The sorting is
       # done so that the enclosing functions are marked before any nested
       # functions.  If the nested functions are changed first the enclosing
       # function won't get marked because it wont match what we are looking for
       # because the function it contains is now marked.
        $fcount = 0;

	# fix for PL110060FME: function calls included in documentation get marked
        # part 1/3
	# idea: escape occurences of ( in the documentation files and 
	#       escape back after the documentation is inserted in the code
	#       therefore we escape first our escape seqeunce so the change
	#	is fully transparent
	# escape char should be no perl meta char and no ()a-zA-Z0-9_
	my $ESC = "!ESC!";
	$tstring =~ s/$ESC/$ESC-0/g;

        # Search the code recursively for matching parenthesis and check if the
        # word preceding them is a function name we are flagging.  If so put the
        # entire string of the function call into a hash(%fun_call).
        get_fn_call($tstring);

        # Marks function calls saves in the %fun_call hash after sorting by
        # value. We need to sort by value (the fcount variable). This is
        # because the search and replace occurs with recursion. It is necessary
        # to modify from the "outside" in. If the script is looking for a
        # particular funtion call that has another function call as a
        # parameter, modifying that inner parameter first would be bad because
        # the script would never find what it is looking for as it has been
        # modified. As a result, we must replace from the initial function
        # call down. "outside in" if you will.
        foreach $key ( sort { $fun_call{$a} <=> $fun_call{$b} } keys %fun_call )
        {
            $key =~ /$funpat/;
            $pattern      = quotemeta($key);
            $lead         = $2;
            $fn_name      = $3;
            $params       = $4;
            $cur_func_doc = '';

            # The pattern is changed to find only full function names with word anchors at the first occurrence
            # of the function name within the pattern to prevent partly marked functions.
            
            $pattern =~ s/$fn_name/\\b$fn_name\\b/;

            # Read the doc file for the current function only
            # if the user like to put the docu in the code
            # and a documentation file is defined for that function.
            if ( ($parm_includedoc) and ( $fcs{$fn_name}->doc_file ) ) {
                if (
                    !open CUR_FCN_DOC,
                    $parm_moduledir
                    . "/port_documentation/port_"
                    . $fcs{$fn_name}->doc_file
                  )
                {

               # We can't open the documentation file. Only show the error once,
               # but log all errors. Set error and record in the log file.
                    if ( $doc_flag != 1 ) {
                        $doc_flag = 1;
                        $$message_parm->{msg_parm_num_messages} =
                          ( $$message_parm->{msg_parm_num_messages} ) +
                          1;    # bump num_messages
                                # set up insert1
                        $$message_parm->{msg_info_array_insert1}[$message_row] =
                          $curr_dir;
                        $$message_parm->{msg_info_array_hashkey}[$message_row] =
                          "DOCUMENTATIONFAIL";
                        $message_row++;    # increment
                    }
                    &log(
"Could not open one or more of the documentation files. Could not open the documentation file \""
                          . $fcs{$fn_name}->doc_file
                          . "\" for function \""
                          . $fn_name
                          . "()\". SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root."
                    );
                }
                else {
                    open( CUR_FCN_DOC,
                            "cat "
                          . $parm_moduledir
                          . "/port_documentation/port_"
                          . $fcs{$fn_name}->doc_file
                          . " |" );
                    my $build;
                    while (<CUR_FCN_DOC>) {
                        $build .= $_;
                    }
                    close CUR_FCN_DOC;

                    $cur_func_doc =
                        "\n\n/* Begin SourceCheckingTool Documentation:\n\n"
                      . $build
                      . "\n\n*/\n\n";
                }
            }    #end if documentation


            # fix for PL110060FME: function calls included in documentation get marked
            # part 2/3
	    $cur_func_doc =~ s/$ESC/$ESC-0/g;
	    $cur_func_doc =~ s/\(/$ESC-1/g;

            # Place the markers for the appropriate level of difficulty around
            # the function name.
            if ( $fcs{$fn_name}->action ne "" ) {
                parameter_scan();
            }
            else {
                if ( $fcs{$fn_name}->diff_level == 1 ) {
                    $tstring =~
s/$pattern/$lead$structs::EASYFLAGFRONT$fn_name$structs::EASYFLAGBACK$cur_func_doc$params/g;
                }
                elsif ( $fcs{$fn_name}->diff_level == 2 ) {
                    $tstring =~
s/$pattern/$lead$structs::MEDIUMFLAGFRONT$fn_name$structs::MEDIUMFLAGBACK$cur_func_doc$params/g;
                }
                elsif ( $fcs{$fn_name}->diff_level == 3 ) {
                    $tstring =~
s/$pattern/$lead$structs::UNKNOWNFLAGFRONT$fn_name$structs::UNKNOWNFLAGBACK$cur_func_doc$params/g;
                }
                elsif ( $fcs{$fn_name}->diff_level == 4 ) {
                    $tstring =~
s/$pattern/$lead$structs::DIFFICULTFLAGFRONT$fn_name$structs::DIFFICULTFLAGBACK$cur_func_doc$params/g;
                }
            }
        }    # end foreach $key ...

        # fix for PL110060FME: function calls included in documentation get marked
        # part 3/3
	$tstring =~ s/$ESC-1/\(/g;
	$tstring =~ s/$ESC-0/$ESC/g;


        #open/create the .portout files for use.

        if ( scalar( keys(%fun_call) ) || $include_file_marked_flag )
        {    #only make a portout file
                #if there are porting issues found
                #(i.e. there are entries in %fun_call)
            if ( !open( P_OUT, ">", $source_file . ".portout" ) ) {

#we couldn't create the portout file. The GUI will fail without the .portout files. and missing one will give incomplete data.
                $master_flag = 1;
                &log(
"Could not create the output file for $source_file. This indicates a permission error for the folder. SourceCheckingTool requires write access to
	the folder."
                );
                $$message_parm->{msg_parm_num_messages} =
                  ( $$message_parm->{msg_parm_num_messages} ) +
                  1;    # increment num_messages
                        # set up insert1
                $$message_parm->{msg_info_array_insert1}[$message_row] =
                  $source_file;
                $$message_parm->{msg_info_array_insert2}[$message_row] =
                  $curr_dir;
                $$message_parm->{msg_info_array_hashkey}[$message_row] =
                  "OUTPUTFAILURE";
                $message_row++;    # increment
                                   #we want to clean up before we leave
                remove_portouts();
                return $master_flag;
            }
            else {
                print P_OUT $tstring;
                close P_OUT;

         # MAKE .PORTOUT FILES HAVE SAME PERMISSIONS AS SOURCE FILES EXCEPT
         # ALSO WRITEABLE FOR THOSE THAT HAVE READ PERMISSION ON THE SOURCE FILE
                $perm_shift = $file_perm & oct(444);
                $file_perm = $file_perm | ( $perm_shift / 2 );
                $chmod = sprintf( "chmod %o $source_file.portout", $file_perm );
                system($chmod);

            }    #end    if(!open (P_OUT, ">", $source_file.".portout"))
        }    #end    if(scalar(keys(%fun_call)))

        $include_file_marked_flag = 0;

        ###############################################################
        #process line by line to get line number info
        ###############################################################

        @lines = split /\n/,
          $tstring;    #break up the huge string on the original new lines
        $line_index = 0;
        $curr_file  = $source_file . ".portout";

        $line_count = scalar(@lines);
        $total_line_count +=
          $line_count;    #record of all lines scanned from all files
        $loop_count = $line_count - 1;    #since the line 1 has an index of 0

        while ( $line_index <= $loop_count ) {    #loop from top to bottom
            if ( $lines[$line_index] =~ m/\#\s*include\s*<\s*!@(\S+)@!\s*>/g )
            { #|| ($lines[$line_index] =~ m/!([@])(\w+)([@])!/g) ){   #check if current line has an include or an unknown function
                $cur_line = $line_index + 1;
                $cur_fcn  = $1;
                $unknown_count++;
                $flagged_line_count++;    #count the flagged lines
                $diff_level   = "To be assessed";
                $flagged_item = "include";
                populate_category_array();
            }
            elsif ( $lines[$line_index] =~ m/\#\s*pragma\s+!([@~&%])(\w+).*/g )
            {
                $cur_line = $line_index + 1;
                $cur_fcn  = $2;
                if ( $1 eq '%' ) {
                    $easy_count++;
                    $diff_level = "Low";
                }
                elsif ( $1 eq '&' ) {
                    $mod_count++;
                    $diff_level = "Medium";
                }
                elsif ( $1 eq '~' ) {
                    $diff_count++;
                    $diff_level = "High";
                }
                elsif ( $1 eq '@' ) {
                    $unknown_count++;
                    $diff_level = "To be assessed";
                }
                $flagged_line_count++;    #count the flagged lines
                $cur_line = $line_index + 1;
                $cur_fcn  = $2;

                $flagged_item = "pragma";
                populate_category_array();
            }
                while ( $lines[$line_index] =~ m/!([%&~@])(\w+)([%&~@])!/g )
                {    #check if current line has a marker in it
                    if ( $1 eq '%' ) {
                        $easy_count++;
                        $diff_level = "Low";
                    }
                    elsif ( $1 eq '&' ) {
                        $mod_count++;
                        $diff_level = "Medium";
                    }
                    elsif ( $1 eq '~' ) {
                        $diff_count++;
                        $diff_level = "High";
                    }
                    elsif ( $1 eq '@' ) {
                        $unknown_count++;
                        $diff_level = "To be assessed";
                    }
                    $flagged_line_count++;    #count the flagged lines
                    $cur_line = $line_index + 1;
                    $cur_fcn  = $2;

                    $flagged_item = "function";
                    populate_category_array();
                }

            $line_index++;
        }


    }    #end foreach(@FILES)

    #sort output to port_categories.out alphabetically
    #except line numbers which are sorted numerically
    #(categories are sorted reverse alphabetically to get Threads to the top)
    @sort_file_out = ();
    @sort_file_out = map { $_->[0] }
    sort {
      @a_fields = @$a[ 1 .. $#$a ];
      @b_fields = @$b[ 1 .. $#$b ];

      $b_fields[0] cmp $a_fields[0]
        || $a_fields[1] cmp $b_fields[1]
        || $a_fields[2] cmp $b_fields[2]
        || $a_fields[3] <=> $b_fields[3]
        }
     map { [ $_, split /:/ ] } @file_out;

     #if we can't open the file to create or append,
     if ( !open( CATEGORIES, ">", "$curr_dir/SCT_files/port_categories.out" )
     )
     {
       $master_flag = 1;
       &log(
         "Could not create/append to the port_categories.out file. Check permissions on the folder $curr_dir/SCT_files/. If
         they are correct, this indicates a possible corruption of the disk itself."
           );
       $$message_parm->{msg_parm_num_messages} =
         ( $$message_parm->{msg_parm_num_messages} ) +
            1;    # increment num_messages
                  # set up insert1
       $$message_parm->{msg_info_array_insert1}[$message_row] =
              "port_categories.out";
       $$message_parm->{msg_info_array_insert2}[$message_row] = $curr_dir;
       $$message_parm->{msg_info_array_hashkey}[$message_row] =
              "OUTPUTFAILURE";
       $message_row++;    # increment
                          #we want to clean up before we leave
       remove_portouts();
       return $master_flag;
    }
      else {
        for ( $counter = 0 ; $counter < $fcn_count ; $counter++ ) {
                print CATEGORIES $sort_file_out[$counter];
        }
        close CATEGORIES;

#remove ????
#	$file_perm = 00666;
#        $chmod = sprintf("chmod %o $curr_dir/SCT_files/port_categories.out", $file_perm);
#        system($chmod);
    }

    #do math on variables to build the metrics file.
    #$flagged_line_count+=$unknown_count;
    #need to make sure we don't do any division by zero.

    if ( $total_line_count > 0 )
      {
        my $bf_flagged = Math::BigFloat->new($flagged_line_count);

        $bf_flagged->copy()->bdiv($total_line_count)->bmul(100)->precision(1);
        $flag_percent = $bf_flagged->bstr();
      }
      else
      {
        $flag_percent = 0;
      }

    if ( $flagged_line_count > 0 )
      {
        my $bf_diff    = Math::BigFloat->new($diff_count);
        my $bf_mod     = Math::BigFloat->new($mod_count);
        my $bf_easy    = Math::BigFloat->new($easy_count);
        my $bf_unknown = Math::BigFloat->new($unknown_count);

        $bf_diff->bdiv($flagged_line_count)->bmul(100)->precision(1);
        $diff_percent = $bf_diff->bstr();

        $bf_mod->bdiv($flagged_line_count)->bmul(100)->precision(1);
        $mod_percent = $bf_mod->bstr();

        $bf_easy->bdiv($flagged_line_count)->bmul(100)->precision(1);
        $easy_percent = $bf_easy->bstr();

        $bf_unknown->bdiv($flagged_line_count)->bmul(100)->precision(1);
        $unknown_percent = $bf_unknown->bstr();

      }
      else
      {
        $diff_percent = 0;
        $mod_percent = 0;
        $easy_percent = 0;
        $unknown_percent = 0;
      }

    #create the metrics file. if we can't, don't even try and write to it
    if ( !open( METRICS, ">", "$curr_dir/SCT_files/port_metrics.out" ) ) {
        $master_flag = 1;
        &log(
"Could not create the port_metrics.out file. Check permission on the folder $curr_dir/SCT_files/. If they are correct,
	this indicates a possible corruption of the disk itself."
        );
        $$message_parm->{msg_parm_num_messages} =
          ( $$message_parm->{msg_parm_num_messages} ) +
          1;    # increment num_messages
                # set up insert1
        $$message_parm->{msg_info_array_insert1}[$message_row] =
          "port_metrics.out";
        $$message_parm->{msg_info_array_insert2}[$message_row] = $curr_dir;
        $$message_parm->{msg_info_array_hashkey}[$message_row] =
          "OUTPUTFAILURE";
        $message_row++;    # increment
    }
    else {
        print METRICS "Base of directory tree scanned: $curr_dir\n\n";
        printf METRICS "Scanned:\t\t %8d lines\n", $total_line_count;
# This is a quick fix for a wrong persentage if more functions are flagged then lines ( percentages > 100 )
#        printf METRICS "Flagged:\t\t %8d functions (%.0f%%)\n\n", $flagged_line_count, $flag_percent;
        printf METRICS "Flagged:\t\t %8d functions\n\n", $flagged_line_count;
        printf METRICS
          "High difficulty:\t %8d functions flagged (%.0f%% of the total)\n",
          $diff_count, $diff_percent;
        printf METRICS
          "Medium difficulty:\t %8d functions flagged (%.0f%% of the total)\n",
          $mod_count, $mod_percent;
        printf METRICS
          "Low difficulty:\t\t %8d functions flagged (%.0f%% of the total)\n",
          $easy_count, $easy_percent;
        printf METRICS
          "To be assessed:\t\t %8d functions flagged (%.0f%% of the total)\n",
          $unknown_count, $unknown_percent;
        if ( $assembler_count > 0 ) {
            print METRICS "\n$assembler_count assembler file(s) found.\n";
        }
        close METRICS;

#remove ???????
#	$file_perm = 00666;
#        $chmod = sprintf("chmod %o $curr_dir/SCT_files/port_metrics.out", $file_perm);
#        system($chmod);
    }

    return $master_flag;

}    #end scanner()

##############################################
#read_funcs($\$)
#read port_funcs.in file and put list of fuctions to flag and other associated information
#into a hash(%fcs) of structs keyed on function name
##############################################
sub read_funcs($\$) {

   #fcs is a hash to hold all the information in the port_funcs.in file for each
   #Solaris OS function. The key is the Solaris OS function name.
    %fcs = ();
    my ( $parm_moduledir, $message_parm ) = @_;    # get the parameter list

    #does the port_funcs file even exist? can't go on without it.
    if ( !( -e "$parm_moduledir" . "/port_ins/port_funcs.in" ) ) {
        $master_flag = 1;
        &log(
"Could not find the port_funcs.in file in the installation directory. SourceCheckingTool may not have been installed correctly. Try reinstalling as root."
        );
        $$message_parm->{msg_parm_num_messages} =
          ( $$message_parm->{msg_parm_num_messages} ) +
          1;                                       # increment num_messages
                                                   # set up insert1 and insert2
        $$message_parm->{msg_info_array_insert1}[$message_row] =
          "port_funcs.in";
        $$message_parm->{msg_info_array_insert2}[$message_row] = $curr_dir;
        $$message_parm->{msg_info_array_hashkey}[$message_row] =
          "CANTFINDANINFILE";
        $message_row++;                            # increment
        return;
    }
    else {

        #open file, and read in. if we can't, we mark the flag and return out.
        if ( !open( FUNCS, "$parm_moduledir" . "/port_ins/port_funcs.in" ) ) {
            $master_flag = 1;
            &log(
"Could not open the port_funcs.in file in the installation directory. SourceCheckingTool may not have been installed correctly. Try reinstalling as root."
            );
            $$message_parm->{msg_parm_num_messages} =
              ( $$message_parm->{msg_parm_num_messages} ) +
              1;    # increment num_messages
                    # set up insert1 and insert2
            $$message_parm->{msg_info_array_insert1}[$message_row] =
              "port_funcs.in";
            $$message_parm->{msg_info_array_insert2}[$message_row] = $curr_dir;
            $$message_parm->{msg_info_array_hashkey}[$message_row] =
              "CANTOPENANINFILE";
            $message_row++;    # increment
            return;
        }
        else {
            while (<FUNCS>) {
                chomp;
                @fcn_element = split /:/, $_;
                $fcs{ $fcn_element[0] } = fcn_info->new();
                $fcs{ $fcn_element[0] }->linux_fcn( $fcn_element[1] );
                $fcs{ $fcn_element[0] }->category( $fcn_element[2] );
                $fcs{ $fcn_element[0] }->action( $fcn_element[3] );
                $fcs{ $fcn_element[0] }->doc_file( $fcn_element[4] );
                $fcs{ $fcn_element[0] }->diff_level( $fcn_element[5] );
            }
            close FUNCS;
        }    #end inner else
    }    #end outer if-else
}    # end read_funcs

##############################################
#read_pragmas($\$)
#read port_pragmas.in file and put list of fuctions to flag and other associated information
#into a hash(%pragma) of structs keyed on pragma name
##############################################
sub read_pragmas($\$) {

#pragma is a hash to hold all the information in the port_pragmas.in file for each
#Solaris OS pragma. The key is the Solaris OS pragma name.
    %pragma = ();
    my ( $parm_moduledir, $message_parm ) = @_;    # get the parameter list

    #does the port_pragmas file even exist? can't go on without it.
    if ( !( -e "$parm_moduledir" . "/port_ins/port_pragmas.in" ) ) {
        $master_flag = 1;
        &log(
"Could not find the port_pragmas.in file in the installation directory. SourceCheckingTool may not have been installed correctly. Try reinstalling as root."
        );
        $$message_parm->{msg_parm_num_messages} =
          ( $$message_parm->{msg_parm_num_messages} ) +
          1;                                       # increment num_messages
                                                   # set up insert1 and insert2
        $$message_parm->{msg_info_array_insert1}[$message_row] =
          "port_pragmas.in";
        $$message_parm->{msg_info_array_insert2}[$message_row] = $curr_dir;
        $$message_parm->{msg_info_array_hashkey}[$message_row] =
          "CANTFINDANINFILE";
        $message_row++;                            # increment
        return;
    }
    else {

        #open file, and read in. if we can't, we mark the flag and return out.
        if ( !open( PRAGMAS, "$parm_moduledir" . "/port_ins/port_pragmas.in" ) )
        {
            $master_flag = 1;
            &log(
"Could not open the port_pragmas.in file in the installation directory. SourceCheckingTool may not have been installed correctly. Try reinstalling as root."
            );
            $$message_parm->{msg_parm_num_messages} =
              ( $$message_parm->{msg_parm_num_messages} ) +
              1;    # increment num_messages
                    # set up insert1 and insert2
            $$message_parm->{msg_info_array_insert1}[$message_row] =
              "port_pragmas.in";
            $$message_parm->{msg_info_array_insert2}[$message_row] = $curr_dir;
            $$message_parm->{msg_info_array_hashkey}[$message_row] =
              "CANTOPENANINFILE";
            $message_row++;    # increment
            return;
        }
        else {
            while (<PRAGMAS>) {
                chomp;
                @fcn_element = split /:/,
                  $_;          #still using fcn_element for pragmas
                $pragma{ $fcn_element[0] } = fcn_info->new();
                $pragma{ $fcn_element[0] }->linux_fcn( $fcn_element[1] );
                $pragma{ $fcn_element[0] }->category( $fcn_element[2] );
                $pragma{ $fcn_element[0] }->action( $fcn_element[3] );
                $pragma{ $fcn_element[0] }->doc_file( $fcn_element[4] );
                $pragma{ $fcn_element[0] }->diff_level( $fcn_element[5] );
            }
            close PRAGMAS;
        }    #end inner else
    }    #end outer if-else
}    # end read_pragmas()

#############################################
#read_incls($\$)
#read port_includes.in file which contains all the .h files from /usr/include down the
#directory tree on a SuSE Linux S/390 system. This is read into a hash(%incls) keyed on file name.
#############################################
sub read_incls($\$) {
    %incls = ();
    my ( $parm_moduledir, $message_parm ) = @_;    # get the parameter list

    #does the port_incls file exisit? can't go on without it.
    if ( !( -e "$parm_moduledir" . "/port_ins/port_incls.in" ) ) {
        $master_flag = 1;
        &log(
"Could not find the port_incls.in file in the installation directory. SourceCheckingTool may not have been installed correctly. Try reinstalling as
	root."
        );
        $$message_parm->{msg_parm_num_messages} =
          ( $$message_parm->{msg_parm_num_messages} ) +
          1;                                       # increment num_messages
                                                   # set up insert1 and insert2
        $$message_parm->{msg_info_array_insert1}[$message_row] =
          "port_incls.in";
        $$message_parm->{msg_info_array_insert2}[$message_row] = $curr_dir;
        $$message_parm->{msg_info_array_hashkey}[$message_row] =
          "CANTFINDANINFILE";
        $message_row++;                            # increment
        return;
    }
    else {

        #open file, and read in. if we can't, we mark the flag and return out.
        if ( !open( INCLS, "$parm_moduledir" . "/port_ins/port_incls.in" ) ) {
            $master_flag = 1;
            &log(
"Could not open the port_incls.in file in the installation directory. SourceCheckingTool may not have been installed correctly. Try reinstalling as
	root."
            );
            $$message_parm->{msg_parm_num_messages} =
              ( $$message_parm->{msg_parm_num_messages} ) +
              1;    # increment num_messages
                    # set up insert1 and insert2
            $$message_parm->{msg_info_array_insert1}[$message_row] =
              "port_incls.in";
            $$message_parm->{msg_info_array_insert2}[$message_row] = $curr_dir;
            $$message_parm->{msg_info_array_hashkey}[$message_row] =
              "CANTOPENANINFILE";
            $message_row++;    # increment
            return;
        }
        else {
            while (<INCLS>) {
                chomp;
                $_ =~ s/\/usr\/include\///;
                $incls{$_} = 1;
            }
            close INCLS;
        }    #end inner else
    }    #end outer if-else
}    # end read_incls()

##############################################
#read_parameters($\$)
#read port_parameters.in file and place the info into the hash %param_check
##############################################
sub read_parameters($\$) {

#param_check is a hash to hold all the information in the port_parameters.in file for each
#function that needs to have their parameters scanned for possible difficulty level change
    %param_check = ();
    my ( $parm_moduledir, $message_parm ) = @_;    # get the parameter list

    #does the port_parameters file even exist? can't go on without it.
    if ( !( -e "$parm_moduledir" . "/port_ins/port_parameters.in" ) ) {
        $master_flag = 1;
        &log(
"Could not find the port_parameters.in file in the installation directory. SourceCheckingTool may not have been installed correctly. Try reinstalling as
	root."
        );
        $$message_parm->{msg_parm_num_messages} =
          ( $$message_parm->{msg_parm_num_messages} ) +
          1;                                       # increment num_messages
                                                   # set up insert1 and insert2
        $$message_parm->{msg_info_array_insert1}[$message_row] =
          "port_parameters.in";
        $$message_parm->{msg_info_array_insert2}[$message_row] = $curr_dir;
        $$message_parm->{msg_info_array_hashkey}[$message_row] =
          "CANTFINDANINFILE";
        $message_row++;                            # increment
        return;
    }
    else {

        #open file, and read in. if we can't, we mark the flag and return out.
        if (
            !open( FUNCS, "$parm_moduledir" . "/port_ins/port_parameters.in" ) )
        {
            $master_flag = 1;
            &log(
"Could not open the port_parameters.in file in the installation directory. SourceCheckingTool may not have been installed correctly. Try reinstalling as
	root."
            );
            $$message_parm->{msg_parm_num_messages} =
              ( $$message_parm->{msg_parm_num_messages} ) +
              1;    # increment num_messages
                    # set up insert1 and insert2
            $$message_parm->{msg_info_array_insert1}[$message_row] =
              "port_parameters.in";
            $$message_parm->{msg_info_array_insert2}[$message_row] = $curr_dir;
            $$message_parm->{msg_info_array_hashkey}[$message_row] =
              "CANTOPENANINFILE";
            $message_row++;    # increment
            return;
        }
        else {
            while (<FUNCS>) {
                my @p_check_array;
                my $cur_element;
                my $number_of_params;
                chomp;
                @fcn_element      = split /:/, $_;
                $number_of_params = $fcn_element[1];
                $cur_element      = 0;
                while ( $cur_element < $number_of_params ) {
                    $p_check_array[$cur_element] = param_info->new();
                    $p_check_array[$cur_element]
                      ->param_slot( $fcn_element[ 2 + $cur_element * 5 ] );
                    $p_check_array[$cur_element]
                      ->param_list1( $fcn_element[ 3 + $cur_element * 5 ] );
                    $p_check_array[$cur_element]
                      ->list1_level( $fcn_element[ 4 + $cur_element * 5 ] );
                    $p_check_array[$cur_element]
                      ->param_list2( $fcn_element[ 5 + $cur_element * 5 ] );
                    $p_check_array[$cur_element]
                      ->list2_level( $fcn_element[ 6 + $cur_element * 5 ] );
                    $cur_element++;
                }
                $param_check{ $fcn_element[0] } = [@p_check_array];
            }
            close FUNCS;
        }    #end inner else
    }    #end outer if-else
}    # end read_parameters

########################################
#remove_portouts(\$)
#this will search the directory tree for all portout files created.
#proceeds to go through and unlink these files
########################################
sub remove_portouts(\$) {
    my ($message_parm) = @_;    #get parameter list
      #if we've been to this folder before, we remove any previous data from the .out files.
    if ( -e "$curr_dir/SCT_files" && glob("$curr_dir/SCT_files/*") ) {
        if ( !unlink glob("$curr_dir/SCT_files/*") ) {
            $master_flag = 1;
            $$message_parm->{msg_parm_num_messages} =
              ( $$message_parm->{msg_parm_num_messages} ) +
              1;    # bump num_messages
            $$message_parm->{msg_info_array_insert1}[$message_row] = $curr_dir;
            $$message_parm->{msg_info_array_hashkey}[$message_row] =
              "SCTFILESDIRPROB";
            $message_row++;    # increment
        }
        else {
            rmdir("$curr_dir/SCT_files");
        }
    }

    open REMOVE_PORTOUTS, "find $curr_dir -name '*.portout' 2>&1 |"
      ;                        #trap any errors from the screen
    @portouts = <REMOVE_PORTOUTS>;
    close REMOVE_PORTOUTS;

    map { chomp; } @portouts;

    foreach (@portouts) {
        if (/^find: /) {
            &log("Possible error when trying to remove old portout files\n$_");
            if ( $find_flag != 1 ) {

#only want to show the message to the screen once, but we want to record all the errors to the log
                $find_flag                              = 1;
                $master_flag                            = 0;
                $$message_parm->{msg_parm_num_messages} =
                  ( $$message_parm->{msg_parm_num_messages} ) +
                  1;    # increment num_messages
                $$message_parm->{msg_info_array_insert1}[$message_row] =
                  $curr_dir;
                $$message_parm->{msg_info_array_hashkey}[$message_row] =
                  "SOMESCANFAILED";
                $message_row++;    # increment
            }
            shift(@portouts)
              ; #if we had find errors, just toss that error out, but log it to the file
        }
        elsif ( !unlink($_) ) {
            if ( $remove_flag != 1 ) {

#only want to show the message to the screen once, but we want to record all the errors to the log
                $remove_flag                            = 1;
                $master_flag                            = 1;
                $$message_parm->{msg_parm_num_messages} =
                  ( $$message_parm->{msg_parm_num_messages} ) +
                  1;    # bump num_messages
                $$message_parm->{msg_info_array_insert1}[$message_row] =
                  $curr_dir;
                $$message_parm->{msg_info_array_hashkey}[$message_row] =
                  "PORTOUTREMOVEFAIL";
                $message_row++;    # increment
            }
            &log(
"Encountered an error attempting to remove the old portout file $_"
            );
        }

    }

}    # end remove_portouts

########################################
#old_portouts()
#this will search the directory tree for old scan results
########################################
sub old_portouts() {
    if ( -e "$curr_dir/SCT_files" ) {
        return 1;
    }
    open REMOVE_PORTOUTS, "find $curr_dir -name '*.portout' |";
    @portouts = <REMOVE_PORTOUTS>;
    close REMOVE_PORTOUTS;
    return scalar(@portouts);
}    # end old_portouts

#########################################
#get_include
#searches the file for any includes that are not listed
#on the machine that is being ported to. flags them as unknown
########################################
sub get_include {
    my (%excludelist) = @_;

    #parse lines to find include files
    my $left_side  = $1;
    my $incl_name  = $2;
    my $right_side = $3;

    $incl_name =~ m/.*?(\w+\.\w+)/;

    if ( !exists( $incls{$incl_name} ) && !exists( $excludelist{$1} ) ) {
        $incl_name =~ s/\+/\\\+/g;
        $temp = $left_side
          . $structs::UNKNOWNFLAGFRONT
          . $incl_name
          . $structs::UNKNOWNFLAGBACK
          . $right_side . "\n";
        $temp =~ s/\\\+/\+/g;
        $include_file_marked_flag = 1;   #there is an include issue in this file
                                         #so create a .portout for it
    }
}    # end get_include

#########################################
#get_pragmas
#searches the file for any pragmas that are not listed
#on the machine that is being ported to. flags them as unknown
########################################
sub get_pragmas($$\$) {
    my ( $parm_moduledir, $parm_includedoc, $message_parm ) = @_;
    my $left_side  = $1;
    my $right_side = $3;
    $fn_name      = $2;
    $cur_func_doc = "";
    if ( exists( $pragma{$fn_name} ) ) {

        # Read the doc file for the current pragma only
        # if the user like to put the docu in the code
        # and a documentation file is defined for that pragma.
        if ( ($parm_includedoc) and ( $pragma{$fn_name}->doc_file ) ) {
            if (
                !open CUR_FCN_DOC,
                $parm_moduledir
                . "/port_documentation/port_"
                . $pragma{$fn_name}->doc_file
              )
            {

#we can't open the documentation file. only show the error once, but log all errors. set error and record in the log file
                if ( $doc_flag != 1 ) {
                    $doc_flag = 1;
                    $$message_parm->{msg_parm_num_messages} =
                      ( $$message_parm->{msg_parm_num_messages} ) +
                      1;    # bump num_messages
                            # set up insert1
                    $$message_parm->{msg_info_array_insert1}[$message_row] =
                      $curr_dir;
                    $$message_parm->{msg_info_array_hashkey}[$message_row] =
                      "DOCUMENTATIONFAIL";
                    $message_row++;    # increment
                }
                &log(
"Could not open one or more of the documentation files. Could not open the documentation file \""
                      . $pragma{$fn_name}->doc_file
                      . "\" for pragma \""
                      . $fn_name
                      . "\". SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root."
                );
            }
            else {
                open( CUR_FCN_DOC,
                        "cat "
                      . $parm_moduledir
                      . "/port_documentation/port_"
                      . $pragma{$fn_name}->doc_file
                      . " |" );
                my $build = "";
                while (<CUR_FCN_DOC>) {
                    $build .= $_;
                }
                close CUR_FCN_DOC;

                $cur_func_doc =
                  "\n\n/* Begin SourceCheckingTool Documentation:\n\n" . $build
                  . "\n\n*/\n\n";
            }
        }    #end if documentation
        $pattern = quotemeta($temp);
        if ( $pragma{$fn_name}->diff_level == 1 ) {
            $temp = $left_side
              . $structs::EASYFLAGFRONT
              . $fn_name
              . $structs::EASYFLAGBACK
              . $right_side
              . $cur_func_doc . "\n";
        }
        elsif ( $pragma{$fn_name}->diff_level == 2 ) {
            $temp = $left_side
              . $structs::MEDIUMFLAGFRONT
              . $fn_name
              . $structs::MEDIUMFLAGBACK
              . $right_side
              . $cur_func_doc . "\n";
        }
        elsif ( $pragma{$fn_name}->diff_level == 4 ) {
            $temp = $left_side
              . $structs::DIFFICULTFLAGFRONT
              . $fn_name
              . $structs::DIFFICULTFLAGBACK
              . $right_side
              . $cur_func_doc . "\n";
        }
    }
    else {
        $temp = $left_side
          . $structs::UNKNOWNFLAGFRONT
          . $fn_name
          . $structs::UNKNOWNFLAGBACK
          . $right_side . "\n";
    }
    $include_file_marked_flag = 1;    #there is a pragma issue in this file
                                      #so create a .portout for it
}    # end get_pragmas

#########################################
#get_fn_call($)
#recursivly finds all matched parenthesis and checks if the word preceding them is a function we are flagging
#if so it saves the function call in a hash
#########################################

sub get_fn_call($);  # prototype, to enable parameter checking in recursive call

sub get_fn_call($) {
    my ($cur_str) = @_;
    my $function_name;

    while ( $cur_str =~ m/$funpat/g ) {

        # The group values resulting from matching with the abouve regular
        # expression are as follows:
        #   $1 will contain the entire function call. It will contain the name
        #      of the function being called, as well as the parameters being
        #      passed
        #   $3 will contain the name of the function being called.
        #   $4 will contain the parameters being passed in the function call
        #      in question.

        if ( $fcs{$3} ) {

            # Remove any leading spaces or operator symbols from the function
            # name as they lead to duplicate entries for the same function and
            # may later confuse the algorithm setting markers.
            $function_name = $1;
            $function_name =~ s/^\W//;
            $fun_call{$function_name} = $fcount;
            $fcount++;
        }
        $4 =~ /^\(((.*\n*.*)*)\)$/;
        get_fn_call($1);
    }
}    # end get_fn_call

#########################################
#populate_category_array()
#Put current flagged functions and includes and associated information into an array(@file_out)
#of appropriately formatted strings(category:function:file name:line number:difficulty level)
#later sorted and printed to the port_categories.out file
#########################################
sub populate_category_array(\$) {
    $curr_file =~ s/$curr_dir\///;
    if ( exists( $fcs{$cur_fcn} ) ) {

        #this is a known function problem
        $file_out[$fcn_count] =
            $fcs{$cur_fcn}->category
          . ":$cur_fcn:$curr_file:"
          . $cur_line
          . ":$diff_level:"
          . $fcs{$cur_fcn}->doc_file . "\n";
        $fcn_count++;
    }
    elsif ( exists( $pragma{$cur_fcn} ) ) {

        #this is a known function problem
        $file_out[$fcn_count] =
            $pragma{$cur_fcn}->category
          . ":$cur_fcn:$curr_file:"
          . $cur_line
          . ":$diff_level:"
          . $pragma{$cur_fcn}->doc_file . "\n";
        $fcn_count++;
    }
    else {

        #this is an include file that has been flagged.
        if ( $flagged_item eq "pragma" ) {
            $file_out[$fcn_count] =
"Pragmas:$cur_fcn:$curr_file:$cur_line:To be assessed:Miscellaneous.txt\n";
        }
        else {
            $file_out[$fcn_count] =
"IncludeFiles:$cur_fcn:$curr_file:$cur_line:To be assessed:IncludeFiles.txt\n";
        }
        $fcn_count++;
    }
}    # end populate_category_array

#########################################
#get_portouts(\$)
#search the directory tree for all the *.portout files we have created
#return an array containing the listing of any files found
# called by the GUI front end to get the list of files
# also returns any messages that need displaying
#########################################
sub get_portouts(\$) {
    my ($message_parm) = @_;    #get parameter list

    #    $portout_flag=0; #reset to 0  to avoid any possible scoping issues
    open GET_PORTOUTS, "find $curr_dir -name '*.portout' 2>&1 |";
    @portouts = <GET_PORTOUTS>;
    close GET_PORTOUTS;
    foreach (@portouts) {

#STDERR is placed at the start of the array. trim off any find errors before the return.
        if (/^find: /) {

#there was an error searching the directory structure. we couldn't open all the dirs to search
            if ( $find_flag != 1 ) {

#only want to mark the error once. and display the error once if we haven't done that already, do so here.
                $find_flag = 1;
                $$message_parm->{msg_parm_num_messages} =
                  ( $$message_parm->{msg_parm_num_messages} ) +
                  1;    # bump num_messages
                $$message_parm->{msg_info_array_insert1}[$message_row] =
                  $curr_dir;
                $$message_parm->{msg_info_array_hashkey}[$message_row] =
                  "SOMESCANFAIL";
                $message_row++;    # increment
            }

            #only show the error once, but log all errors
            &log(
"Could not scan all folders under $curr_dir. The following error was encountered\n$_"
            );
            shift(@portouts);
        }
    }
    return @portouts;
}    # end get_portouts

sub parameter_scan() {
    my $loop_count = 0;
    my $family;
    my $inner_count;
    my $plist1;
    my $plist2;
    my $listval1;
    my $listval2;
    my $cur_level = 0;

#if we are looking at the first or last parameter, we don't want the ( or ) in the variable.
    my $clean_params = $params;

    #remove the ( and ) and any spaces
    $clean_params =~ s/\(//g;
    $clean_params =~ s/\)//g;
    $clean_params =~ s/\s*//g;

    my @breakup = split /,/, $clean_params;

    #find where in the array we are going to look for a possible problem

    while ( $loop_count < scalar( @{ $param_check{$fn_name} } ) ) {
        my $index = $param_check{$fn_name}[$loop_count]->param_slot - 1;
        my @value = split /\|/, $breakup[$index];
        $inner_count = 0;
        $plist1      = $param_check{$fn_name}[$loop_count]->param_list1;
        $plist2      = $param_check{$fn_name}[$loop_count]->param_list2;

        while ( $inner_count < scalar(@value) ) {
            if (   $plist1 =~ /$value[$inner_count],/
                || $plist1 =~ /$value[$inner_count]\z/ )
            {
                $listval1 = $param_check{$fn_name}[$loop_count]->list1_level;
                if ( ($listval1) > $cur_level ) {
                    $cur_level = $listval1;
                }
            }
            elsif ($plist2 =~ /$value[$inner_count],/
                || $plist2 =~ /$value[$inner_count]\z/ )
            {
                $listval2 = $param_check{$fn_name}[$loop_count]->list2_level;
                if ( ($listval2) > $cur_level ) {
                    $cur_level = $listval2;
                }
            }
            else {
                if ( $cur_level < 3 ) {
                    $cur_level = 3;
                }
            }
            $inner_count++;
        }
        $loop_count++;
    }

    if ( $cur_level == 1 ) {
        $tstring =~
s/$pattern/ $structs::EASYFLAGFRONT$fn_name$structs::EASYFLAGBACK$cur_func_doc$params/g;
    }

    elsif ( $cur_level == 2 ) {
        $tstring =~
s/$pattern/ $structs::MEDIUMFLAGFRONT$fn_name$structs::MEDIUMFLAGBACK$cur_func_doc$params/g;
    }

    elsif ( $cur_level == 3 ) {
        $tstring =~
s/$pattern/ $structs::UNKNOWNFLAGFRONT$fn_name$structs::UNKNOWNFLAGBACK$cur_func_doc$params/g;
    }

    elsif ( $cur_level == 4 ) {
        $tstring =~
s/$pattern/ $structs::DIFFICULTFLAGFRONT$fn_name$structs::DIFFICULTFLAGBACK$cur_func_doc$params/g;
    }

}    #end parameter_scan()

1;
