##################################################
# SourceCheckingTool
# Version: 1.0.0
#
# (C) Copyright International Business Machines (IBM), 2001, 2002, 2005
# Useable and distributable only with IBM permission and under IBM terms and
# conditions. It may not be copied or further distributed unless authorized in
# writing by IBM.
#
##################################################
package portManager;

BEGIN {
    my $openfile;
    my $moduleplusdir = "/SCT/port_code_scan.pm";
    my $mylibpath;

    push @INC, "/opt/ibm/mksl/SCT";
  PATH: foreach (@INC) {
        $openfile = open( SCANPM, "$_" . "$moduleplusdir" );
        if ($openfile) {
            $mylibpath = $_ . '/SCT';
            print "Using files in " . $mylibpath . "\n";
            unshift @INC, $mylibpath;
            close(SCANPM);
            last PATH;
        }
    }
    if ( !$openfile ) {
        print "Unable to open $moduleplusdir"
          . ". SourceCheckingTool may not have been installed correctly. Try reinstalling Source Checking Tool as root.\n";
        exit;
    }
}

use 5.8.0;    # require perl 5.8.0 or later, issue error message otherwise
# use strict;
# no warnings 'redefine';
# use diagnostics -verbose;

use Tk::BrowseEntry;
use Tk;
use Tk::DirTree;
use Tk::Tree;
use Cwd;
use Tk::TextUndo;
use Tk::Pane;
use Tk::Dialog;
use Tk::ProgressBar;
use port_code_scan;
use port_messages;
use port_structs;

# Misc global variables
our $ON               = 1;
our $OFF              = 0;
our $INIT             = 2;
our $DEBUG            = $OFF;
our $FIRSTCALL        = $OFF;
our $SUMMARYFIRSTCALL = $OFF;
our $FNAME            = "";
our $curr_dir         = Cwd::cwd();
our $LAUNCHDIR        = Cwd::cwd();
our $MODULEDIR        = '';
our $LOGFILE;
our $PORTMINSTBASEDIR = '/SCT';
our $LOGFILENAME      = '/SCT_files/sct.log';
our $CATFILE;
our $CATFILENAME = '/SCT_files/port_categories.out';
our $METRICSFILE;
our $METRICSFILENAME = '/SCT_files/port_metrics.out';
our $MSGFILE         = '/port_ins/port_message_table.in';
our $openFile;
our $DOCFILESDIR  = "port_documentation";
our $portHelpDir  = "/port_help";
our $portDocDir   = "/port_documentation";
our $portInsDir   = "/port_ins";
our $contentFile  = "/port_help/port_help_toc.doc";
our $guiFile      = "/port_help/port_help_gui.doc";
our $viewFile     = "/port_help/port_help_views.doc";
our $editorFile   = "/port_help/port_help_editor.doc";
our $messageFile  = "/port_help/port_help_message.doc";
our $buildFile    = "/port_help/port_help_build.doc";
our $aboutFile    = "/port_help/port_about.doc";
our $guiLabel     = "Using Source Checking Tool";
our $viewLabel    = "  Views";
our $editorLabel  = "  Editor functions";
our $messageLabel = "Messages";
our $buildLabel   = "Build and Compile Considerations";
our $funcs        = "/port_ins/port_funcs.in";
our $incls        = "/port_ins/port_incls.in";
our $parameters   = "/port_ins/port_parameters.in";
our $messageTable = "/port_ins/port_message_table.in";
our $pragmas      = "/port_ins/port_pragmas.in";
our $scanMod      = '/port_code_scan.pm';
our $logReturn    = 0;

our $DOCEXT      = '.txt';
our $GZEXT       = '';
our $PORTMPREFIX = "port_";
our $PORTOUTEXT  = ".portout";
our $OKCOLOR     = 'black';
our $WARNCOLOR   = 'red';
our $OLDCURSOR   = ' ';
our $vitals      = 0;
our $METRICSBASEDIR;
our $METRICSSCANFILES;
our $METRICSFLAGGED;
our $HIGHPERCENT;
our $MEDIUMPERCENT;
our $LOWPERCENT;
our $UNKNOWNPERCENT;
our $POSTSCRIPTHEIGHT = 600;
our $POSTSCRIPTWIDTH  = 600;
our %DOCHASH;

# variables to keep track of the current view
our $PORTOUTVIEW         = 0;
our $SUMMARYVIEW         = 1;
our $METRICSVIEW         = 2;
our $GRAPHICSVIEW        = 3;
our $CURRENTVIEW         = $SUMMARYVIEW;
our $VIEWGET             = "";
our $SAVEASBEGINLABEL    = "Save As...";
our $SAVEASPORTOUTLABEL  = "Save Portout List As...";
our $SAVEASSUMMARYLABEL  = "Save Summary As...";
our $SAVEASGRAPHICSLABEL = "Save Graphics As...";
our $SAVEASMETRICSLABEL  = "Save Metrics As...";

# varaiables to keep track of call types
our $CALLTYPE;
our $SCANTYPE    = 0;
our $DISPLAYTYPE = 1;
our $SAVEASTYPE  = 2;
our $PORTOUTTYPE = 3;
our $SUMMARYTYPE = 4;
our $METRICTYPE  = 5;
our $GRAPHICTYPE = 6;

# variables to keep track of main function calls
our $SCANCALL    = $OFF;
our $DISPLAYCALL = $OFF;

# arrays to keep track of views
our @PORTOUTS;
our @SUMMARY;
our @SUMMARYSAVEAS;
our @METRICS;
our @GRAPHICS;

# variable for whether to include documentation
our $INCLUDEDOC = $OFF;

# Edit flags
our $PORTMANAGERDOC    = "/* SourceCheckingTool Documentation:";
our $PORTMANAGERENDDOC = "\\*/";
our $ENDLINE           = "\\n";

# Windows and window structures
our $mw;
our $width;
our $height;
our $xpos;
our $ypos;
our $mbutton;
our $file_mbutton;
our $view_mbutton;
our $help_mbutton;
our $statusText;
our $statusLine;
our $tx1;
our $portoutFrame;
our $summaryFrame;
our $metricsFrame;
our $graphicsFrame;
our $w_list;
our $w_title;
our $fw;
our $tree;
our $textArea;
our $summary_title;
our $summary_menu;
our $helpPanel;
our $helpFrame;
our $helpList;
our $helpText;
our $helpOK;
our $helpSelection;
our $ok;
our $metricsList;
our $metricsTitle;
our $metricsw;
our $metricsText;
our $graphicsCanvas;
our $graphicsTitle;
our $graphicsw;

# Message component variables
our $insert1;      # message insert
our $insert2;      # message insert
our $msg_index;    # index into message table
our %messages;     # hash
our $message_parm =
  message_parm_struct->new();    # message parameter to be passed to backend
our @message_info_array =
  message_info_array_struct->new();    # array with message information
our $message_row = 0;                  # initialize

$message_parm->{msg_parm_message_info_array_struct} = @message_info_array;
$message_parm->{msg_parm_num_messages}              = 0;

################################################
# Check if the user wants to debug the program #
# Alert them as to where the output is going   #
################################################
if ( exists( $ARGV[0] ) and ( $ARGV[0] eq "-debug" ) ) {
    print "debug mode....\n";
    $DEBUG = $ON;
}

$vitals = &initCheckVitals;
if ( $vitals == 1 ) {

    # We got a problem. Most likely installation
    # Messages will be printed from initCheckVitals
    exit;
}

###############################################
# Build the Main Window                       #
###############################################
$mw     = MainWindow->new();
$width  = int( $mw->screenwidth() * .6 );
$height = int( $mw->screenheight() * .6 );
$xpos   = int( $width / 15 );
$ypos   = int( $height / 22 );
$mw->geometry("$width\Lx$height+$xpos+$ypos");
$mw->resizable( 1, 1 );
$mw->title("SourceCheckingTool");
$mw->configure( -relief => 'groove' );
$mbutton = $mw->Frame(
    -borderwidth => 3,
    -relief      => 'groove',
)->pack( '-side' => 'top', -fill => 'x' );

###########################
# Create File Menu button #
###########################
$file_mbutton = $mbutton->Menubutton(
    -text => "File",

    #-state => 'disabled',
    -menuitems => [
        [
            'command' => "Scan...",

            #-state => 'disabled',
            "-command" => sub {
                $CALLTYPE = $SCANTYPE, &scan,;
            },
            "-underline" => 0
        ],
        "-",
        [
            'command'  => "Display...",
            "-command" => sub {
                $CALLTYPE = $DISPLAYTYPE, &display;
            },
            "-underline" => 0
        ],
        "-",
        [
            'command'  => $SAVEASBEGINLABEL,
            "-state"   => "disabled",
            "-command" => sub {
                $CALLTYPE = $SAVEASTYPE, &saveAs;
            },
            "-underline" => 0
        ],
        "-",
        [
            'command'    => "Exit",
            "-command"   => sub { exit(0) },
            "-underline" => 0
        ]
    ],
    -tearoff => 0
)->pack( -side => 'left' );

###############################
# Create the View Menu button #
###############################
$view_mbutton = $mbutton->Menubutton(
    -text => "View",

    #-state => 'disabled',
    -menuitems => [
        [
            'Radiobutton',
            'Portouts',
            "-command" => sub {
                $CALLTYPE = $PORTOUTTYPE, &viewPortouts;
            },
            "-underline"   => 0,
            "-selectcolor" => 'blue',
            "-indicatoron" => 1
        ],
        "-",
        [
            'Radiobutton',
            "Summary",
            "-command" => sub {
                $CALLTYPE = $SUMMARYTYPE, &viewSummary;
            },
            "-underline"   => 0,
            "-selectcolor" => 'blue'
        ],
        "-",
        [
            'Radiobutton',
            "Metrics",
            "-command" => sub {
                $CALLTYPE = $METRICTYPE, &viewMetrics;
            },
            "-underline"   => 0,
            "-selectcolor" => 'blue'
        ],
        "-",
        [
            'Radiobutton',
            "Graphics",
            "-command" => sub {
                $CALLTYPE = $GRAPHICTYPE, &viewGraphics;
            },
            "-underline"   => 0,
            "-selectcolor" => 'blue'
        ]
    ],
    -tearoff => 0
)->pack( -side => 'left' );

#################################################
# Pick the default selection for the Radio buttons
# so we start with something
#################################################
$VIEWGET = $view_mbutton->cget('-menu');
$FIRSTCALL = $INIT;    # Let them know no processing. We're just seeting the
                       # darn thing
$VIEWGET->invoke("Summary");
$FIRSTCALL = $OFF;

######################################
# Create the Help Menu Button
######################################
$help_mbutton = $mbutton->Menubutton(
    -text      => "Help",
    -menuitems => [
        [
            'command'    => "Help",
            "-command"   => \&help,
            "-underline" => 0
        ],
        [
            'command'    => "About",
            "-command"   => \&about,
            "-underline" => 0
        ]
    ],
    -tearoff => 0
)->pack( -side => 'right' );

#####################################
# Create the status area
#####################################
$statusLine = $mw->Label(
    -textvariable => \$statusText,
    -relief       => 'ridge',
    -background   => 'grey',
    -anchor       => 'w'
  )->pack(
    -side => 'bottom',
    -fill => 'x'
  );

MainLoop;    # End of main menu loop

################################################################
# Wrapper for the scanner function in port_code_scan.pm
################################################################
sub scanner_wrapper($$$$) {

    #first check if old files are available
    if ( old_portouts() != 0 ) {
        my $yesno = $mw->DialogBox(
            -title   => "old portout files found",
            -buttons => [ "Yes", "No" ]
        );
        $yesno->add(
            'Label',
            -anchor  => 'w',
            -justify => 'left',
            -text    => qq(
It seems that this directory has been scanned before. Do
you really want to scan again and remove all old portout
files?)
        )->pack;
        my $answer = $yesno->Show();
        if ( $answer eq "No" ) {
            return 1;
        }
    }

    #proceed scanning
    my $close_requested = 0;
    my $progressFrame = $mw->Toplevel( -title => "Scanning files" );
    $progressFrame->geometry('400x120');

    my $progressBar = $progressFrame->ProgressBar(
        -from => '0',
        -to   => '100',
        -gap  => '0'
      )->pack(
        -fill   => 'x',
        -expand => 1,
        -padx   => 20,
        -pady   => 20
      );

    my $progressText =
      $progressFrame->Label( -text => "0/0 files" )->pack( -fill => 'x' );

    my $filenameText = $progressFrame->Label()->pack( -fill => 'x' );

    my $cancelButton = $progressFrame->Button(
        -text    => "Cancel",
        -command => sub { $close_requested = 1 }
    )->pack();

    my $scanReturn = scanner(
        $_[0],
        $_[1],
        $_[2],
        $_[3],
        sub {
            if ( Tk::Exists($progressBar) ) {
                $progressBar->value( $_[1] * 100 / $_[2] );
                $progressText->configure( -text => "$_[1]/$_[2] files" );
                $filenameText->configure( -text => "$_[0]" );
                $progressFrame->update();
                return ($close_requested);
            }
        }
    );

    if ( Tk::Exists($progressFrame) ) {
        $progressFrame->destroy();
    }
    return $scanReturn;
}

################################################################
# Build a new window that let's the user pick the base directory
# and displayes the information depending on the view
################################################################
sub scan {

    # call the directory window
    # return if user cancels
    my $dir = getDir();
    if ( $dir eq "Cancel" ) {
        return;
    }

    if ( $DEBUG == $ON ) {
        &log("scan: In scan subroutine......");
    }

    # We start fresh. After a scan we kill all widgets
    &cleanWidgets;

    # Check the view we're in
    if ( $CURRENTVIEW == $PORTOUTVIEW ) {
        &viewPortouts;
    }
    if ( $CURRENTVIEW == $SUMMARYVIEW ) {
        &viewSummary;
    }
    if ( $CURRENTVIEW == $METRICSVIEW ) {
        &viewMetrics;
    }
    if ( $CURRENTVIEW == $GRAPHICSVIEW ) {
        &viewGraphics;
    }
    return 0;
}

################################################################
# Build a new window that let's the user pick the base directory
# and displayes the information depending on the view
################################################################
sub display {

    if ( $DEBUG == $ON ) {
        &log("dspl: In scan display sub routine");
    }

    # call the directory window
    # return if user cancels
    my $dir = getDir();
    if ( $dir eq "Cancel" ) {
        return;
    }

    # We start fresh after a display we kill all widgets
    &cleanWidgets;

    # Check the view we're in
    if ( $CURRENTVIEW == $PORTOUTVIEW ) {
        &viewPortouts;
    }
    if ( $CURRENTVIEW == $SUMMARYVIEW ) {
        &viewSummary;
    }
    if ( $CURRENTVIEW == $METRICSVIEW ) {
        &viewMetrics;
    }
    if ( $CURRENTVIEW == $GRAPHICSVIEW ) {
        &viewGraphics;
    }
    return 0;
}

################################################################
# Present the user with an interface to select a directory
################################################################
sub getDir {
    if ( $DEBUG == $ON ) {
        &log("gtdr: In getDir subroutine......");
    }
    my $t = $mw->Toplevel;
    $t->title("Scan Base Directory");
    my $ok = 0;    # flag: "1" means OK, "-1" means cancelled

    # Create Frame widget before the DirTree widget, so it's always visible
    # if the window gets resized.
    my $f = $t->Frame->pack( -fill => "x", -side => "bottom" );

    my $d;
    $d = $t->Scrolled(
        'DirTree',
        -scrollbars      => 'osoe',
        -width           => 35,
        -height          => 20,
        -selectmode      => 'browse',
        -exportselection => 1,
        -browsecmd       => sub { $curr_dir = shift },

        # open a directory. Selection is only possible with
        # the Ok button.
        -command => sub {
            if ( $d->getmode(@_) eq 'close' ) {
                $d->close(@_);
            }
            else {
                if ( $d->getmode(@_) eq 'open' ) {
                    $d->open(@_);
                }
            }
        },
    )->pack( -fill => "both", -expand => 1 );

    # Set the initial or current directory
    $d->chdir($curr_dir);
    if ($DEBUG) {
        &log("gtdr: Initial Directory -> $curr_dir");
    }

    # Set the doc selection button
    if ( $CALLTYPE == $SCANTYPE ) {
        $f->Checkbutton(
            -text        => 'Include Documentation',
            -variable    => \$INCLUDEDOC,
            -selectcolor => 'blue'
        )->pack;
    }

    # Set the selection buttons
    $f->Button(
        -text    => '  Ok  ',
        -command => sub { $ok = 1 }
    )->pack( -side => 'left', -padx => 40 );
    $f->Button(
        -text    => 'Cancel',
        -command => sub { $ok = -1 }
    )->pack( -side => 'left', -padx => 40 );

    #Wait for some input
    $f->waitVariable( \$ok );

    # If we got a directory then return the directory selected
    if ( $ok == 1 ) {
        $t->destroy;    # We got what we wanted kill the directory window
        if ($DEBUG) {
            &log("gtdr: OK button selected");
            &log("gtdr: Directory Selected -> $curr_dir");
        }

        # See if directory can be read
        if ( !( -r "$curr_dir" ) ) {
            $insert1 = $curr_dir;
            &display_message("ROOTSCANDIRPROB");
            return "Cancel";
        }

        # See if we can write to directory
        if ( !( -w "$curr_dir" ) ) {
            $insert1 = $curr_dir;
            &display_message("ROOTSCANDIRPROB");
            return "Cancel";
        }

        return $curr_dir;
    }

    # if cancel was hit let's go back where we came from
    if ( $ok == -1 ) {
        $t->destroy;
        if ($DEBUG) {
            &log("gtdr: Cancel button selected");
        }
        return "Cancel";
    }
    return 0;
}

################################################################
# Display the selected portout file
################################################################
sub displayFile {

    my $docfileopen;
    my $filename;

    if ($DEBUG) {
        &log("dspf: In displayFile subroutine");
        &log("dspf: File selected: $FNAME");
    }

    if ( Exists($tx1) ) {
        $tx1->destroy();
    }

    # Remove blanks
    chomp($FNAME);
    $filename = $FNAME;

    # We're going to do some work. Let user know
    # Change status line and then cursor

    &display_message("PLEASEWAIT");
    $statusLine->update;
    &workCursor;
    $mw->update;

    # Look for the file
    if ( !( -e "$FNAME" ) ) {
        $insert1 = $FNAME;
        &display_message("FILENOTFOUND");
        &log("dspf: Could find $FNAME. Check that file exists.");
        &restoreCursor;

        # Something is very wrong. Base directory may be out of sync
        &display_message("OUTOFSYNCH");
        &log(
            "vsrp: Could not find $filename. Base directory may be out of sync."
        );
        &display_message("BLANK");
        return;
    }

    # Try to read and load file.
    $docfileopen = open( PORTOUTFILE, $FNAME );

    if ( $docfileopen == 1 ) {
        $tx1 = $portoutFrame->Scrolled( qw/TextUndo -scrollbars osoe/,
            -wrap => 'word' );
        $tx1->tagConfigure(
            'high',
            foreground => 'black',
            background => 'red'
        );
        $tx1->tagConfigure(
            'medium',
            foreground => 'black',
            background => 'yellow'
        );
        $tx1->tagConfigure(
            'low',
            foreground => 'black',
            background => 'green'
        );
        $tx1->tagConfigure(
            'unknown',
            foreground => 'black',
            background => 'white'
        );
        $tx1->tagConfigure(
            'unknownmark',
            foreground => 'black',
            background => 'white'
        );
        $tx1->tagConfigure( 'doctag', foreground => 'red' );
        my $lineNumb = 0;
        my $index1   = 0;
        my $index2   = 0;
        my $beginIndex1;
        my $beginIndex2;
        my $endIndex1;
        my $endIndex2;
        my $docIndicator = 0;
        my $tempDoc;
        my $firstString;
        my $lastString;
        my $stringLength;
        my $textRetrieve;
        my $startRetrieve;
        my $endRetrieve;

        while (<PORTOUTFILE>) {

            ####################################################################################
# Check each line for eye catchers for marking.
# Take into account that markings may be more than one or different on the same line
# This version doesn't handle multiple line tags
            ####################################################################################
            $lineNumb = $lineNumb + 1;

            # Put the line
            $tx1->insert( "end", $_ );

            # Check for difficult tags
            if ( $_ =~ /$structs::DIFFICULTFLAGFRONT/ ) {
                while ( $_ =~ m/$structs::DIFFICULTFLAGFRONT/g ) {
                    $index1 = pos $_;
                    $_ =~ m/$structs::DIFFICULTFLAGBACK/g;
                    $index2 = ( pos $_ ) - 2;
                    $tx1->tagAdd( 'high', "$lineNumb.$index1",
                        "$lineNumb.$index2" );

                    # delete tags
                    $beginIndex1 = $index1 - 2;
                    $beginIndex2 = $index1;
                    $endIndex1   = $index2;
                    $endIndex2   = $index2 + 2;
                    $tx1->delete( "$lineNumb.$endIndex1",
                        "$lineNumb.$endIndex2" );
                    $tx1->delete( "$lineNumb.$beginIndex1",
                        "$lineNumb.$beginIndex2" );
                    $_ = $tx1->get( "$lineNumb.0", "end" );
                    $tx1->tagBind(
                        'high',
                        "<Double-Button-1>",
                        sub {
                            $startRetrieve = $tx1->index('insert wordstart');
                            $endRetrieve   = $tx1->index('insert wordend');
                            $textRetrieve  =
                              $tx1->get( $startRetrieve, $endRetrieve );
                            &getTextDoc($textRetrieve);

                        }
                    );
                    $tx1->tagBind( 'high', "<Any-Enter>",
                        sub { shift->configure( -cursor => 'hand2' ) } );
                    $tx1->tagBind( 'high', "<Any-Leave>",
                        sub { shift->configure( -cursor => 'xterm' ) } );
                }
            }

            # Check for moderate flags
            if ( $_ =~ /$structs::MEDIUMFLAGFRONT/ ) {
                while ( $_ =~ m/$structs::MEDIUMFLAGFRONT/g ) {
                    $index1 = pos $_;
                    $_ =~ m/$structs::MEDIUMFLAGBACK/g;
                    $index2 = ( pos $_ ) - 2;
                    $tx1->tagAdd( 'medium', "$lineNumb.$index1",
                        "$lineNumb.$index2" );

                    # delete tags
                    $beginIndex1 = $index1 - 2;
                    $beginIndex2 = $index1;
                    $endIndex1   = $index2;
                    $endIndex2   = $index2 + 2;
                    $tx1->delete( "$lineNumb.$endIndex1",
                        "$lineNumb.$endIndex2" );
                    $tx1->delete( "$lineNumb.$beginIndex1",
                        "$lineNumb.$beginIndex2" );
                    $_ = $tx1->get( "$lineNumb.0", "end" );
                    $tx1->tagBind(
                        'medium',
                        "<Double-Button-1>",
                        sub {
                            $startRetrieve = $tx1->index('insert wordstart');
                            $endRetrieve   = $tx1->index('insert wordend');
                            $textRetrieve  =
                              $tx1->get( $startRetrieve, $endRetrieve );
                            &getTextDoc($textRetrieve);

                        }
                    );
                    $tx1->tagBind( 'medium', "<Any-Enter>",
                        sub { shift->configure( -cursor => 'hand2' ) } );
                    $tx1->tagBind( 'medium', "<Any-Leave>",
                        sub { shift->configure( -cursor => 'xterm' ) } );
                }
            }

            # Check for easy flags
            if ( $_ =~ /$structs::EASYFLAGFRONT/ ) {
                while ( $_ =~ m/$structs::EASYFLAGFRONT/g ) {
                    $index1 = pos $_;
                    $_ =~ m/$structs::EASYFLAGBACK/g;
                    $index2 = ( pos $_ ) - 2;
                    $tx1->tagAdd( 'low', "$lineNumb.$index1",
                        "$lineNumb.$index2" );

                    # delete tags
                    $beginIndex1 = $index1 - 2;
                    $beginIndex2 = $index1;
                    $endIndex1   = $index2;
                    $endIndex2   = $index2 + 2;
                    $tx1->delete( "$lineNumb.$endIndex1",
                        "$lineNumb.$endIndex2" );
                    $tx1->delete( "$lineNumb.$beginIndex1",
                        "$lineNumb.$beginIndex2" );
                    $_ = $tx1->get( "$lineNumb.0", "end" );
                    $tx1->tagBind(
                        'low',
                        "<Double-Button-1>",
                        sub {
                            $startRetrieve = $tx1->index('insert wordstart');
                            $endRetrieve   = $tx1->index('insert wordend');
                            $textRetrieve  =
                              $tx1->get( $startRetrieve, $endRetrieve );
                            &getTextDoc($textRetrieve);

                        }
                    );
                    $tx1->tagBind( 'low', "<Any-Enter>",
                        sub { shift->configure( -cursor => 'hand2' ) } );
                    $tx1->tagBind( 'low', "<Any-Leave>",
                        sub { shift->configure( -cursor => 'xterm' ) } );
                }
            }

            # Check for unknown flags
            if ( $_ =~ /$structs::UNKNOWNFLAGFRONT/ ) {
                while ( $_ =~ m/$structs::UNKNOWNFLAGFRONT/g ) {
                    $index1 = pos $_;
                    $_ =~ m/$structs::UNKNOWNFLAGBACK/g;
                    $index2 = ( pos $_ ) - 2;
                    $tx1->tagAdd( 'unknown', "$lineNumb.$index1",
                        "$lineNumb.$index2" );

                    # delete tags
                    $beginIndex1 = $index1 - 2;
                    $beginIndex2 = $index1;
                    $endIndex1   = $index2;
                    $endIndex2   = $index2 + 2;
                    $tx1->delete( "$lineNumb.$endIndex1",
                        "$lineNumb.$endIndex2" );
                    $tx1->delete( "$lineNumb.$beginIndex1",
                        "$lineNumb.$beginIndex2" );
                    $_             = $tx1->get( "$lineNumb.0", "end" );
                    $startRetrieve = $tx1->index('insert wordstart');
                    $endRetrieve   = $tx1->index('insert wordend');
                    $textRetrieve  = $tx1->get( $startRetrieve, $endRetrieve );

                    if ( $DOCHASH{$textRetrieve} ne "" ) {

                        # we clip the tags we have to account for that
                        $index1 = $index1 - 2;
                        $index2 = $index2 - 2;
                        $tx1->tagAdd( 'unknownmark', "$lineNumb.$index1",
                            "$lineNumb.$index2" );
                        $tx1->tagBind(
                            'unknownmark',
                            "<Double-Button-1>",
                            sub {
                                $startRetrieve =
                                  $tx1->index('insert wordstart');
                                $endRetrieve  = $tx1->index('insert wordend');
                                $textRetrieve =
                                  $tx1->get( $startRetrieve, $endRetrieve );
                                &getTextDoc($textRetrieve);
                            }
                        );
                        $tx1->tagBind( 'unknownmark', "<Any-Enter>",
                            sub { shift->configure( -cursor => 'hand2' ) } );
                        $tx1->tagBind( 'unknownmark', "<Any-Leave>",
                            sub { shift->configure( -cursor => 'xterm' ) } );
                    }
                }
            }

            if ($docIndicator) {
                $tempDoc = $_;

  # find the first character. Then add the length to first index to find the end
  # of the line. We need this to get the indexes
  # also make sure this is not a blank line with a */ otherwise TK error on
  # $_ =~ m/$firstString/g;
                if ( $tempDoc ne $ENDLINE ) {
                    $tx1->tagAdd( 'doctag', "$lineNumb.0",
                        "$lineNumb." . ( length($tempDoc) - 1 ) );
                }
                if ( $_ =~ m/$PORTMANAGERENDDOC/g ) {

                    # Get out of the loop and delete the end delimeters
                    $docIndicator = 0;
                    $index1       = ( pos $_ ) - 2;
                    $index2       = $index1 + 2;
                    $tx1->delete( "$lineNumb.$index1", "$lineNumb.$index2" );
                }
            }

            if ( $_ =~ /$PORTMANAGERDOC/ ) {

                # Find the end of line character and delete to one less
                # Don't delete the whole line otherwise this will not work
                $_ =~ m/$ENDLINE/g;
                $index2 = ( pos $_ ) - 1;
                $tx1->delete( "$lineNumb.0", "$lineNumb.$index2" );
                $docIndicator = 1;
            }
        }    #while

        $insert1 = $filename;
        &display_message("FILELOADED");
    }
    else {
        $insert1 = $filename;
        $insert2 = $curr_dir . $LOGFILENAME;
        &display_message("CRITICALOPENERROR");
        &log(
"dspf: Could not open $filename. Check that file exists or permission exists to read the file."
        );
        &restoreCursor;

        # Something is very wrong. Base directory may be out of sync
        &display_message("OUTOFSYNCH");
        &log(
            "vsrp: Could not find $filename. Base directory may be out of sync."
        );
        &display_message("BLANK");
        return;
    }

    $tx1->ResetUndo();
    $tx1->pack( -side => 'right', -fill => 'both', -expand => 1 );
    
    &restoreCursor;
    close(PORTOUTFILE);
}

#############################################################
# Routine to view the portouts files when portouts is
# selected from the view menu or display selected and view is
# the default selection
#############################################################
sub viewPortouts {

    if ($DEBUG) {
        &log("vipo: In viewPortouts subroutine");
    }

    my $file_index;
    my $scanReturn;
    my $catopen;
    my $prob;
    my $func;
    my $file;
    my $fileline;
    my $level;
    my $docline;
    my @firstpass;
    my $record;
    my $m = 0;

    #print "In viewPortout\n";
    # Return if we're just initializing
    if ( $FIRSTCALL == $INIT ) {
        if ( $FIRSTCALL == $OFF ) {
            $file_mbutton->entryconfigure(
                $SAVEASBEGINLABEL,
                -state => 'disabled',
                -label => $SAVEASPORTOUTLABEL
            );
        }
        return;
    }

    # view check then we have to make sure we have done a scan or
    # display first

    if ( $CALLTYPE == $PORTOUTTYPE ) {
        if ( $FIRSTCALL == $OFF ) {
            &display_message("NODISPLAY");
            &disablePortoutLabel;
            $CURRENTVIEW = $PORTOUTVIEW;
            return;
        }

        # We don't know we'll succeed but at least we know where we are
        &disablePortoutLabel;
        $CURRENTVIEW = $PORTOUTVIEW;
    }

    # We're going to do some work. Let user know
    # Change status line and then cursor
    &display_message("PLEASEWAIT");
    $statusLine->update;
    &workCursor;
    $mw->update;

    # Scan called us we gather info again
    if ( $CALLTYPE == $SCANTYPE ) {
        splice( @PORTOUTS, 0, $#PORTOUTS + 1 );    # kill the array
        $scanReturn =
          scanner_wrapper( $curr_dir, $INCLUDEDOC, $MODULEDIR, $message_parm );
        if ( ( $message_parm->{msg_parm_num_messages} ) > 0 ) {
            &process_messages($message_parm);
        }
        if ( $scanReturn == 0 ) {
            @PORTOUTS = get_portouts($message_parm);
            if ( ( $message_parm->{msg_parm_num_messages} ) > 0 ) {
                &process_messages($message_parm);
            }
        }
        else {
            $FIRSTCALL = $OFF;

            # Do all the things when we're not displaying anything
            # Make sure we get the right label according to the view
            $FIRSTCALL = $OFF;
            &disablePortoutLabel;
            &display_message("BLANK");
            &restoreCursor;
            return;
        }
    }

    # At this point we should have a categories file
    $CATFILE = $curr_dir . $CATFILENAME;
    chomp($CATFILE);

    # Look for the file
    if ( !( -e "$CATFILE" ) ) {
        $insert1 = $CATFILE;
        &display_message("FILENOTFOUND");
        &log("vsrp: Could not find $CATFILE. Check that file exists.");
        if ( $CALLTYPE == $SCANTYPE ) {
            &display_message("NOPROBFOUND");
        }
        else {
            &display_message("NODISPLAY");
        }
        &disablePortoutLabel;
        $CURRENTVIEW = $PORTOUTVIEW;
        &restoreCursor;
        &cleanWidgets;
        $FIRSTCALL = $OFF;
        return;
    }

    # Read the categories input file
    $catopen = open( CATFILE, $CATFILE );

    %DOCHASH = ();
    if ( $catopen == 1 ) {
        while ( $record = <CATFILE> ) {
            $firstpass[$m] = $record;

            # Build the function name documentation hash
            ( $prob, $func, $file, $fileline, $level, $docline ) =
              split( /:/, $record );
            if ( $docline ne '' ) {
                $DOCHASH{"$func"} = $docline;
            }
            $m = $m + 1;
        }    #while $record
    }
    else {
        $insert1 = $CATFILE;
        $insert2 = $curr_dir . $LOGFILENAME;
        &log(
"vsrp: Could not open $CATFILE. Check that file exists or permission exists to read the file."
        );
        &display_message("CRITICALOPENERROR");
        if ( $CALLTYPE == $SCANTYPE ) {
            &display_message("NOPROBFOUND");
        }
        else {
            &display_message("NODISPLAY");
        }
        &disablePortoutLabel;
        $CURRENTVIEW = $PORTOUTVIEW;
        &restoreCursor;
        &cleanWidgets;
        $FIRSTCALL = $OFF;
        return;
    }

    close(CATFILE);

    if ( $m == 0 ) {

        # Something is not right.
        if ( $CALLTYPE == $SCANTYPE ) {
            &display_message("NOPROBFOUND");
        }
        else {
            &display_message("NODISPLAY");
        }
        $FIRSTCALL = $OFF;
        &disablePortoutLabel;
        &restoreCursor;
        $CURRENTVIEW = $PORTOUTVIEW;
        return;
    }

    &hideWidgets;

    # Display called us we gather info again
    # without a full scan
    if ( $CALLTYPE == $DISPLAYTYPE || $CALLTYPE == $PORTOUTTYPE ) {
        splice( @PORTOUTS, 0, $#PORTOUTS + 1 );    # kill the array
        $file_index = 0;
        @PORTOUTS   = get_portouts($message_parm);
        if ( ( $message_parm->{msg_parm_num_messages} ) > 0 ) {
            &process_messages($message_parm);
        }
    }

    &restoreCursor;

    if ($DEBUG) {
        foreach (@PORTOUTS) {
            &log("scan: File Detected -> $_");
        }
    }

    if ( $PORTOUTS[0] ne '' ) {

        my $displayportouts = displayPortouts();

        if ( $displayportouts == 0 ) {

            # we got something so we can now save
            # Make sure we get the right label according to the view
            &enablePortoutLabel;

            #Who call us. Thus the message
            if ( $CALLTYPE == $SCANTYPE ) {
                &display_message("SCANDONE");
            }

            if ( $CALLTYPE == $PORTOUTTYPE ) {
                &display_message("PORTVIEW");
            }

            if ( $CALLTYPE == $DISPLAYTYPE ) {
                &display_message("DISPLAYDONE");
            }

            #If we were able to display we say we have succeded on a call
            $FIRSTCALL = $ON;
        }
        else {
            if ( $CALLTYPE == $SCANTYPE ) {
                &display_message("NOPROBFOUND");
            }
            else {
                &display_message("NODISPLAY");
            }
        }
    }    # if @PORTOUTS ne @null
    else {
        if ( $CALLTYPE == $SCANTYPE ) {
            &display_message("NOPROBFOUND");
        }
        else {
            &display_message("NODISPLAY");
        }
    }

    # Set the current view
    $CURRENTVIEW = $PORTOUTVIEW;

}    # END SUB viewPortouts

###########################################################
# Routine to destroy the portouts view
###########################################################
sub destroyPortoutView {
    if ($DEBUG) {
        &log("dpov: In destroyPortoutView subroutine");
    }

    #is widget on the screen?
    if ( Exists($portoutFrame) ) {
        $portoutFrame->destroy;
    }
}

###########################################################
# Routine to destroy the summary view
###########################################################
sub destroySummaryView {
    if ($DEBUG) {
        &log("dsmv: In destroySummaryView subroutine");
    }

    #Everything is wrap in the frame. Kill them all
    if ( Exists($summaryFrame) ) {
        $summaryFrame->destroy();
    }
}

###########################################################
# Routine to destroy the metrics view
###########################################################
sub destroyMetricsView {
    if ($DEBUG) {
        &log("dmtv: In destroyMetricsView subroutine");
    }

    #is widget on the screen?
    if ( Exists($metricsFrame) ) {
        $metricsFrame->destroy;
    }
}

###########################################################
# Routine to destroy the graphics view
###########################################################
sub destroyGraphicsView {
    if ($DEBUG) {
        &log("dgpv: In destroyGraphicsView subroutine");
    }

    #Everything is wrap in the frame. Kill them all
    if ( Exists($graphicsFrame) ) {
        $graphicsFrame->destroy();
    }
}

###########################################################
# Routine to hide the summary view
###########################################################
sub hideSummaryView {
    if ($DEBUG) {
        &log("hsmv: In hideSummaryView subroutine");
    }

    #Everything is wrap in the frame. Do a disapearing act
    if ( Exists($summaryFrame) ) {
        $summaryFrame->packForget();
    }
}

###########################################################
# Routine to hide the portouts view
###########################################################
sub hidePortoutView {
    if ($DEBUG) {
        &log("hpov: In hidePortoutView subroutine");
    }

    #is widget on the screen?
    if ( Exists($portoutFrame) ) {
        $portoutFrame->packForget();
    }
}

###########################################################
# Routine to hide the metrics view
###########################################################
sub hideMetricsView {
    if ($DEBUG) {
        &log("hmtv: In hideMetricsView subroutine");
    }

    #is widget on the screen?
    if ( Exists($metricsFrame) ) {
        $metricsFrame->packForget();
    }
}

###########################################################
# Routine to hide the graphics view
###########################################################
sub hideGraphicsView {
    if ($DEBUG) {
        &log("hgpv: In hideGraphicsView subroutine");
    }

    #is widget on the screen?
    if ( Exists($graphicsFrame) ) {
        $graphicsFrame->packForget();
    }
}

###########################################################
# Routine to destroy all the widgets
###########################################################
sub cleanWidgets {
    if ($DEBUG) {
        &log("clwg: In cleanWidgets subroutine");
    }
    &destroyPortoutView;
    &destroySummaryView;
    &destroyMetricsView;
    &destroyGraphicsView;
}

###########################################################
# Routine to hide all the widgets
###########################################################
sub hideWidgets {
    if ($DEBUG) {
        &log("hlwg: In cleanWidgets subroutine");
    }
    &hidePortoutView;
    &hideSummaryView;
    &hideMetricsView;
    &hideGraphicsView;
}

###########################################################
# Routine to display the portouts array
###########################################################
sub displayPortouts {
    if ($DEBUG) {
        &log("dppo: In cleanWidgets subroutine");
    }

    if ( Exists($portoutFrame) ) {

        # This means no new scan or no new display
        # Just redisplay
        $portoutFrame->pack( -fill => 'both', -expand => 1 );
        return;
    }

    $portoutFrame = $mw->Frame();
    $w_list       = $portoutFrame->ScrlListbox(
        -relief     => 'raised',
        -setgrid    => 'no',
        -selectmode => 'single',
        -scrollbars => 'osoe',
    );
    $tx1 = $portoutFrame->Scrolled(qw/TextUndo -scrollbars osoe/);

    foreach (@PORTOUTS) {

        # Take the new line character out. Otherwise it gets
        # displayed
        chomp($_);
        $w_list->insert( "end", $_ );
    }

    $w_list->bind(
        "<Double-Button-1>",
        sub {
            ( $FNAME = ( $w_list->get( $w_list->curselection() ) ) ),
              &displayFile;
        }
    );
    $w_title = $portoutFrame->Label(
        -text       => "Portout Files - Base directory scanned: " . $curr_dir,
        -relief     => 'raised',
        -background => 'grey'
      )->pack(
        -side   => 'top',
        -anchor => 'w',
        -fill   => 'x'
      );

    $w_list->packAdjust(
        -side   => 'left',
        -expand => 'yes',
        -fill   => 'both',
        -delay  => '1'
    );
    $tx1->pack( -side => 'right', -fill => 'both', -expand => 1 );

    $portoutFrame->pack( -fill => 'both', -expand => 1 );

    return 0;
}

###########################################################
# Routine to save a particular output depending on the view
###########################################################
sub saveAs {
    if ($DEBUG) {
        &log("svas: In saveAs subroutine");
    }

    my $open;
    my $stamp;
    my $time;
    my $day;
    my $month;
    my $date;
    my $mytime;
    my $year;
    my $file;

    ###################
    # get the file name
    ###################
    if ( $CURRENTVIEW == $GRAPHICSVIEW ) {

        # Give them a clue this will be a postscript file
        $file = $mw->getSaveFile(
            -title     => "Save As",
            -filetypes => [ [ 'Postscript Files', '.ps' ] ]
        );
    }
    else {
        my @types = ( [ "All Files", "*" ] );
        $file = $mw->getSaveFile(
            -title     => "Save As",
            -filetypes => [ [ 'Any file type', '.txt' ] ]
        );    # Wg
    }

    if ( $file eq '' ) {

        # Cancel was picked. Return nothing to do
        return;
    }

    # They want us to save something open file for writing
    $open = open( OUTFILE, ">$file" );

    if ( $open != 1 ) {

        # Houston we got a problem
        if ( $CURRENTVIEW == $PORTOUTVIEW ) {
            $insert1 = "Portout Report";
        }

        if ( $CURRENTVIEW == $SUMMARYVIEW ) {
            $insert1 = "Summary Report";
        }

        if ( $CURRENTVIEW == $METRICSVIEW ) {
            $insert1 = "Metrics Report";
        }

        if ( $CURRENTVIEW == $GRAPHICSVIEW ) {
            $insert1 = "Graphics Report";
        }

        $insert2 = $file;
        &display_message("WRITEERROR");
        &log(
"svas: Could not open $file. Check that file exists or permission exists to open the file."
        );
        return;
    }

    $stamp = time();
    $time  = localtime($stamp);

    ( $day, $month, $date, $mytime, $year ) = split( /\s+/, $time );

    # Make time better looking
    $time = $day . " " . $month . " " . $date . " " . $year . " " . $mytime;

    if ( $CURRENTVIEW == $PORTOUTVIEW ) {
        $insert1 = "Portout Report";

        # They want to save the portouts
        print OUTFILE "                  SourceCheckingTool Portout Report\n";
        print OUTFILE "                  -----------------------------\n\n";
        print OUTFILE "Time stamp      : ", $time,     "\n";
        print OUTFILE "Search directory: ", $curr_dir, "\n\n";
        foreach (@PORTOUTS) {
            print OUTFILE "$_\n";
        }
    }
    if ( $CURRENTVIEW == $SUMMARYVIEW ) {
        $insert1 = "Summary Report";

        # Let's create a summary report
        print OUTFILE "                  SourceCheckingTool Summary Report\n";
        print OUTFILE "                  ------------------------------\n\n";
        print OUTFILE "Time stamp      : ", $time,     "\n";
        print OUTFILE "Search directory: ", $curr_dir, "\n\n";
        foreach (@SUMMARYSAVEAS) {
            print OUTFILE "$_\n";
        }
    }
    if ( $CURRENTVIEW == $METRICSVIEW ) {
        $insert1 = "Metrics Report";

        # Let's create a metrics report
        print OUTFILE "                  SourceCheckingTool Metrics Report\n";
        print OUTFILE "                  ------------------------------\n\n";
        print OUTFILE "Time stamp      : ", $time,     "\n";
        print OUTFILE "Search directory: ", $curr_dir, "\n\n";
        foreach (@METRICS) {
            print OUTFILE "$_\n";
        }
    }

    if ( $CURRENTVIEW == $GRAPHICSVIEW ) {
        $insert1 = "Graphics Report";

        # Let's create a graphics report
        $graphicsCanvas->postscript(
            -file   => "$file",
            -height => $POSTSCRIPTHEIGHT,
            -width  => $POSTSCRIPTWIDTH
        );
    }

    # Say we're successful
    close(OUTFILE);
    $insert2 = $file;
    &display_message("FILEWRITTEN");
}

###########################################################
# Routine to display the summary view
###########################################################
sub viewSummary {

    my $file_index;
    my $scanReturn;

    if ($DEBUG) {
        &log("vsrp: In viewSummary subroutine");
    }

    # Return if we're just initializing
    if ( $FIRSTCALL == $INIT ) {
        $CURRENTVIEW = $SUMMARYVIEW;
        $file_mbutton->entryconfigure(
            $SAVEASBEGINLABEL,
            -state => 'disabled',
            -label => $SAVEASSUMMARYLABEL
        );
        return;
    }

    # Give apperance of work
    &workCursor;

    &display_message("PLEASEWAIT");
    $mw->update();

    # Do a disapearing act but don't want to kill it.
    # We want to keep the view intact in case they move and return
    &hideWidgets;

    # Check for summary view calls. We have to make sure we have done a scan or
    # display first
    if ( $CALLTYPE == $SUMMARYTYPE ) {
        if ( $FIRSTCALL == $OFF ) {
            &disableSummaryLabel;
            $CURRENTVIEW = $SUMMARYVIEW;
            &restoreCursor;
            &display_message("NODISPLAY");
            return;
        }
    }

# Scan called us we gather info since we want to create the categories file again
    if ( $CALLTYPE == $SCANTYPE ) {
        splice( @PORTOUTS, 0, $#PORTOUTS + 1 );    # kill the array
        $file_index = 0;                           # located on code_scan.pm
        $scanReturn =
          scanner_wrapper( $curr_dir, $INCLUDEDOC, $MODULEDIR, $message_parm );
        if ( ( $message_parm->{msg_parm_num_messages} ) > 0 ) {
            &process_messages($message_parm);
        }
        if ( $scanReturn == 0 ) {
            @PORTOUTS = get_portouts($message_parm);
            if ( ( $message_parm->{msg_parm_num_messages} ) > 0 ) {
                &process_messages($message_parm);
            }
        }
        else {

            # Do all the things when we're not displaying anything
            # Make sure we get the right label according to the view
            $FIRSTCALL = $OFF;
            &disableSummaryLabel;
            &display_message("BLANK");
            &restoreCursor;
            $CURRENTVIEW = $SUMMARYVIEW;
            return;
        }
    }

    if ( $CALLTYPE == $SUMMARYTYPE && Exists($summaryFrame) ) {

        # We don't want to do much work we already did
        # Besides we want to keep the look where we left off
        $summaryFrame->pack( -fill => 'both', -expand => 1 );
        &display_message("PORTVIEW");

        # Do restore the Save As button
        &enableSummaryLabel;
    }
    else {

        # We do this if a display or scan call or if this is our first call
        # init all the things we need to juggle
        my $i = 0;
        my $j = 0;
        my $k = 0;
        my $l = 0;
        my $m = 0;
        my @firstpass;
        my $probHolder;
        my $prob;
        my $funcHolder;
        my $func;
        my $file;
        my $fileline;
        my $docline;
        my $level;
        my $probeasyct    = 0;
        my $probmodct     = 0;
        my $probdiffct    = 0;
        my $probunknownct = 0;
        my $funceasyct    = 0;
        my $funcmodct     = 0;
        my $funcdiffct    = 0;
        my $funcunknownct = 0;
        my $newprob       = 0;
        my $newfunc       = 0;
        my $saveprobeasyct;
        my $saveprobmodct;
        my $saveprobdiffct;
        my $saveprobunknownct;
        my $prevfile;
        my $prevfileline;
        my $prevlevel;
        my $saveprob;
        my $probheadsa;
        my $record;
        my $funcheadsa;
        my $linelevel;
        my $prevprob;
        my $prevfunc;
        my $fileheadsa;
        my $lineheadsa;
        my $catopen;

        # clear the SUMMARY Array. Otherwise these could cause problems
        # if the first search is bigger than second.
        @SUMMARY = ();

        # Check to see if we got something to display
        @PORTOUTS = get_portouts($message_parm);
        if ( ( $message_parm->{msg_parm_num_messages} ) > 0 ) {
            &process_messages($message_parm);
        }

        if ( $PORTOUTS[0] eq '' ) {
            if ( $CALLTYPE == $SCANTYPE ) {
                &display_message("NOPROBFOUND");
            }
            else {
                &display_message("NODISPLAY");
            }
            &disableSummaryLabel;
            &restoreCursor;
            $CURRENTVIEW = $SUMMARYVIEW;
            $FIRSTCALL   = $OFF;
            return;
        }

        $CATFILE = $curr_dir . $CATFILENAME;
        chomp($CATFILE);

        # Look for the file
        if ( !( -e "$CATFILE" ) ) {
            $insert1 = $CATFILE;
            &display_message("MISSINGCATFILE");
            if ( $DEBUG == $ON ) {
                &log("vsrp: Could not find $CATFILE. Check that file exists.");
            }
            if ( $CALLTYPE == $SCANTYPE ) {
                &display_message("NOPROBFOUND");
            }
            else {
                &display_message("NODISPLAY");
            }
            &disableSummaryLabel;
            &restoreCursor;
            &cleanWidgets;
            $FIRSTCALL   = $OFF;
            $CURRENTVIEW = $SUMMARYVIEW;
            return;
        }

        # Read the categories input file
        $catopen = open( CATFILE, $CATFILE );

        # Check for return code. Die statement has bad side effects with the GUI
        %DOCHASH = ();
        if ( $catopen == 1 ) {
            while ( $record = <CATFILE> ) {
                $firstpass[$m] = $record;

                # Build the function name documentation hash
                ( $prob, $func, $file, $fileline, $level, $docline ) =
                  split( /:/, $record );
                if ( $docline ne '' ) {
                    $DOCHASH{"$func"} = $docline;
                }
                $m = $m + 1;
            }    #while $record
        }
        else {
            $insert1 = $CATFILE;
            $insert2 = $curr_dir . $LOGFILENAME;
            &display_message("CRITICALOPENERROR");
            if ( $DEBUG == $ON ) {
                &log(
"vsrp: Could not open $CATFILE. Check that file exists or permission exists to read the file."
                );
            }
            if ( $CALLTYPE == $SCANTYPE ) {
                &display_message("NOPROBFOUND");
            }
            else {
                &display_message("NODISPLAY");
            }
            &disableSummaryLabel;
            &restoreCursor;
            &cleanWidgets;
            $FIRSTCALL   = $OFF;
            $CURRENTVIEW = $SUMMARYVIEW;
            return;
        }

        close(CATFILE);

        if ( $m == 0 ) {

            # Something is not right.
            if ( $CALLTYPE == $SCANTYPE ) {
                &display_message("NOPROBFOUND");
            }
            else {
                &display_message("NODISPLAY");
            }
            $FIRSTCALL = $OFF;
            &disableSummaryLabel;
            &restoreCursor;
            &cleanWidgets;
            $CURRENTVIEW = $SUMMARYVIEW;
            return;
        }

        ##############################################################################
   # Here's the main idea for this loop. First we start searching
   # and keeping count of the problem at hand while keeping a count on it
   # on number and level of difficulty.
   # Once the problem changes then we recursively start looking at the functions
   # up to the point where we stop looking when the problem changed.
   # We keep a count on functions and level of difficulty. When the function
   # changes we have a total for that particular function. We can generate info
   # for that function. We look at the functions this way until we reach
   # the level where we start looking at another problem. We keep doing this
   # and building the array till the end of the file.
   # Main point here is recursion.
        ##############################################################################
        $firstpass[$m] = "::::";
        while ( $i <= $m ) {
            ( $prob, $func, $file, $fileline, $level ) =
              split( /:/, $firstpass[$i] );

            # Check for a corrupted file
            if ( $i < $m ) {
                if (   $prob eq ''
                    || $func     eq ''
                    || $fileline eq ''
                    || $level    eq '' )
                {
                    $insert1 = $CATFILE;
                    &display_message("CORRUPTEDCAT");
                    if ( $CALLTYPE == $SCANTYPE ) {
                        &display_message("NOPROBFOUND");
                    }
                    else {
                        &display_message("NODISPLAY");
                    }
                    $FIRSTCALL = $OFF;
                    &disableSummaryLabel;
                    &restoreCursor;
                    &cleanWidgets;
                    $CURRENTVIEW = $SUMMARYVIEW;
                    return;
                }
            }

            # every new problem has to be counted and saved to detect its change
            if ( $newprob == 0 ) {
                $probHolder = $prob;
                $newprob    = 1;
                $j          = $i;
                if ( $level =~ /Low/ ) {
                    $probeasyct = $probeasyct + 1;
                }
                if ( $level =~ /Medium/ ) {
                    $probmodct = $probmodct + 1;
                }
                if ( $level =~ /High/ ) {
                    $probdiffct = $probdiffct + 1;
                }
                if ( $level =~ /To be assessed/ ) {
                    $probunknownct = $probunknownct + 1;
                }
                $i = $i + 1;
            }
            else {

                # Check if the problem changed on us
                if ( $probHolder ne $prob ) {

                    # We have a different problem. We need to retrack
                    my $firstfunc = 0;
                    $saveprobeasyct    = $probeasyct;
                    $saveprobmodct     = $probmodct;
                    $saveprobdiffct    = $probdiffct;
                    $saveprobunknownct = $probunknownct;
                    $saveprob          = $probHolder;
                    $probeasyct        = 0;
                    $probmodct         = 0;
                    $probdiffct        = 0;
                    $probunknownct     = 0;

                    # Different problem loop
                    # we do it until we reach the point where the problem
                    # changed on us and we start were the last problem left off
                    while ( $j <= $i ) {
                        ( $prob, $func, $file, $fileline, $level ) =
                          split( /:/, $firstpass[$j] );

                        # we follow the same mentality for function
                        # we save and count the first function detected
                        if ( $newfunc == 0 ) {
                            $funcHolder = $func;
                            if ( $level =~ /Low/ ) {
                                $funceasyct = $funceasyct + 1;
                            }
                            if ( $level =~ /Medium/ ) {
                                $funcmodct = $funcmodct + 1;
                            }
                            if ( $level =~ /High/ ) {
                                $funcdiffct = $probdiffct + 1;
                            }
                            if ( $level =~ /To be assessed/ ) {
                                $funcunknownct = $probunknownct + 1;
                            }
                            $newfunc = 1;
                            $j       = $j + 1;
                        }
                        else {
                            if ( $funcHolder ne $func ) {

                        # We got a different function. We got everything we need
                        # build array up to this function
                                my $probhead;
                                my $funchead;
                                my $recordhead;
                                my $filehead;
                                my $linehead;

                                # Different function loop
                                while ( $k < $j ) {
                                    ( $prob, $func, $file, $fileline, $level ) =
                                      split( /:/, $firstpass[$k] );

                                    (
                                        $prevprob, $prevfunc,
                                        $prevfile, $prevfileline,
                                        $prevlevel
                                      )
                                      = split( /:/, $firstpass[ $k - 1 ] );

                                    #######################################
                                    # Build the different output tree heads
                                    # Two arrays are kept. One for the GUI
                                    # and one in case they want to save to
                                    # a file
                                    #######################################
                                    $probhead   = $saveprob . '<';
                                    $probheadsa = $saveprob . '<';

                                    if ( $saveprobeasyct > 0 ) {
                                        $probhead =
                                          $probhead . $saveprobeasyct . '-Low';
                                        $probheadsa = $probheadsa
                                          . $saveprobeasyct . '-Low';
                                    }

                                    if ( $saveprobmodct > 0 ) {
                                        if ( $saveprobeasyct > 0 ) {
                                            $probhead   = $probhead . ',';
                                            $probheadsa = $probheadsa . ',';
                                        }
                                        $probhead = $probhead
                                          . $saveprobmodct
                                          . '-Medium';
                                        $probheadsa = $probheadsa
                                          . $saveprobmodct
                                          . '-Medium';
                                    }

                                    if ( $saveprobdiffct > 0 ) {
                                        if (   $saveprobeasyct > 0
                                            || $saveprobmodct > 0 )
                                        {
                                            $probhead   = $probhead . ',';
                                            $probheadsa = $probheadsa . ',';
                                        }
                                        $probhead =
                                          $probhead . $saveprobdiffct . '-High';
                                        $probheadsa = $probheadsa
                                          . $saveprobdiffct . '-High';
                                    }

                                    if ( $saveprobunknownct > 0 ) {
                                        if (   $saveprobeasyct > 0
                                            || $saveprobmodct > 0
                                            || $saveprobdiffct > 0 )
                                        {
                                            $probhead   = $probhead . ',';
                                            $probheadsa = $probheadsa . ',';
                                        }
                                        $probhead = $probhead
                                          . $saveprobunknownct
                                          . '-To be assessed';
                                        $probheadsa = $probheadsa
                                          . $saveprobunknownct
                                          . '-To be assessed';
                                    }

                                    $probhead   = $probhead . '>';
                                    $probheadsa = $probheadsa . '>';

                                    # Function head build up
                                    $funchead   = $probhead . ":" . $func . '<';
                                    $funcheadsa = "  " . $func . '<';

                                    if ( $funceasyct > 0 ) {
                                        $funchead =
                                          $funchead . $funceasyct . '-Low';
                                        $funcheadsa =
                                          $funcheadsa . $funceasyct . '-Low';
                                    }

                                    if ( $funcmodct > 0 ) {
                                        if ( $funceasyct > 0 ) {
                                            $funchead   = $funchead . ',';
                                            $funcheadsa = $funcheadsa . ',';
                                        }
                                        $funchead =
                                          $funchead . $funcmodct . '-Medium';
                                        $funcheadsa =
                                          $funcheadsa . $funcmodct . '-Medium';
                                    }

                                    if ( $funcdiffct > 0 ) {
                                        if ( $funceasyct > 0 || $funcmodct > 0 )
                                        {
                                            $funchead   = $funchead . ',';
                                            $funcheadsa = $funcheadsa . ',';
                                        }
                                        $funchead =
                                          $funchead . $funcdiffct . '-High';
                                        $funcheadsa =
                                          $funcheadsa . $funcdiffct . '-High';
                                    }

                                    if ( $funcunknownct > 0 ) {
                                        if (   $funceasyct > 0
                                            || $funcmodct > 0
                                            || $funcdiffct > 0 )
                                        {
                                            $funchead   = $funchead . ',';
                                            $funcheadsa = $funcheadsa . ',';
                                        }
                                        $funchead = $funchead
                                          . $funcunknownct
                                          . '-To be assessed';
                                        $funcheadsa = $funcheadsa
                                          . $funcunknownct
                                          . '-To be assessed';
                                    }
                                    $funchead   = $funchead . '>';
                                    $funcheadsa = $funcheadsa . '>';

                                    $filehead = $funchead . ":" . $file;

                                    $fileheadsa = "    " . $file;

                                    if ( $level =~ /Low/ ) {
                                        $linelevel = "<Low>";
                                    }

                                    if ( $level =~ /Medium/ ) {
                                        $linelevel = "<Medium>";
                                    }

                                    if ( $level =~ /High/ ) {
                                        $linelevel = "<High>";
                                    }

                                    if ( $level =~ /To be assessed/ ) {
                                        $linelevel = "<To be assessed>";
                                    }

                                    $linehead =
                                        $filehead . ':' . 'Line-'
                                      . $fileline
                                      . $linelevel;
                                    $lineheadsa =
                                        '      ' . 'Line-'
                                      . $fileline
                                      . $linelevel;

                                    if ( $newprob == 1 ) {

                                # First time we build problem and function nodes
                                        $newprob = 0;
                                        $newfunc = 0;

                                        # not the first time we need to jump
                                        # otherwise we overide
                                        if ( $l != 0 ) {
                                            $l = $l + 1;
                                        }
                                        $SUMMARY[$l]       = $probhead;
                                        $SUMMARYSAVEAS[$l] = $probheadsa;

                                        $l                 = $l + 1;
                                        $SUMMARY[$l]       = $funchead;
                                        $SUMMARYSAVEAS[$l] = $funcheadsa;

                                        $l                 = $l + 1;
                                        $SUMMARY[$l]       = $filehead;
                                        $SUMMARYSAVEAS[$l] = $fileheadsa;

                                        $l                 = $l + 1;
                                        $SUMMARY[$l]       = $linehead;
                                        $SUMMARYSAVEAS[$l] = $lineheadsa;
                                    }
                                    else {
                                        if ( $newfunc == 1 ) {
                                            $newfunc           = 0;
                                            $l                 = $l + 1;
                                            $SUMMARY[$l]       = $funchead;
                                            $SUMMARYSAVEAS[$l] = $funcheadsa;

                                            $l                 = $l + 1;
                                            $SUMMARY[$l]       = $filehead;
                                            $SUMMARYSAVEAS[$l] = $fileheadsa;

                                            $l                 = $l + 1;
                                            $SUMMARY[$l]       = $linehead;
                                            $SUMMARYSAVEAS[$l] = $lineheadsa;
                                        }
                                        else {
                                            if ( $prevfile ne $file ) {

                                                # We build file and line nodes
                                                $l                 = $l + 1;
                                                $SUMMARY[$l]       = $filehead;
                                                $SUMMARYSAVEAS[$l] =
                                                  $fileheadsa;

                                                $l                 = $l + 1;
                                                $SUMMARY[$l]       = $linehead;
                                                $SUMMARYSAVEAS[$l] =
                                                  $lineheadsa;
                                            }
                                            else {

                                 # Only file lines and avoid duplicates
                                 # this will give us a  duplicate node and the
                                 # tree will break. Example is when you have the
                                 # same function on the same line
                                                if (
                                                    $prevfileline ne $fileline )
                                                {
                                                    $l = $l + 1;
                                                    $SUMMARY[$l] = $linehead;
                                                    $SUMMARYSAVEAS[$l] =
                                                      $lineheadsa;
                                                }
                                            }
                                        }
                                    }
                                    $k = $k + 1;
                                }    #while $k
                                $funceasyct    = 0;
                                $funcmodct     = 0;
                                $funcdiffct    = 0;
                                $funcunknownct = 0;
                                $firstfunc     = 0;

                                # get out of loop
                                if ( $j == $i ) {
                                    $j = $i + 1;
                                }
                            }    # funcHolder if
                            else {

                                # We still on the same function
                                if ( $level =~ /Low/ ) {
                                    $funceasyct = $funceasyct + 1;
                                }
                                if ( $level =~ /Medium/ ) {
                                    $funcmodct = $funcmodct + 1;
                                }
                                if ( $level =~ /High/ ) {
                                    $funcdiffct = $funcdiffct + 1;
                                }
                                if ( $level =~ /To be assessed/ ) {
                                    $funcunknownct = $funcunknownct + 1;
                                }
                                $j = $j + 1;
                            }
                        }    #else
                    }    # while $j <= $i
                }    # if probhold ne prob
                else {

                    # Problem still the same
                    if ( $level =~ /Low/ ) {
                        $probeasyct = $probeasyct + 1;
                    }
                    if ( $level =~ /Medium/ ) {
                        $probmodct = $probmodct + 1;
                    }
                    if ( $level =~ /High/ ) {
                        $probdiffct = $probdiffct + 1;
                    }
                    if ( $level =~ /To be assessed/ ) {
                        $probunknownct = $probunknownct + 1;
                    }
                    $i = $i + 1;
                }
            }    #else #$newprob
        }    #while $i

        ##############################################################
        # Build the summary view look
        #############################################################
        $summaryFrame = $mw->Frame()->pack( -fill => 'both', -expand => 1 );

        $summary_title = $summaryFrame->Label(
            -text       => "Summary - Base directory scanned: " . $curr_dir,
            -relief     => 'raised',
            -background => 'grey'
          )->pack(
            -side   => 'top',
            -anchor => 'w',
            -fill   => 'x'
          );

        $tree = $summaryFrame->Scrolled(
            qw/Tree -separator : -exportselection 1
              -scrollbars osoe
              /
        );

        $tree->packAdjust( -side => 'left', -fill => 'both', -delay => 1 );

        $textArea = $summaryFrame->Scrolled(qw/TextUndo -scrollbars osoe/);
        $textArea->pack( -side => 'left', -fill => 'both', -expand => 1 );

        # Populate the tree
        foreach my $d (@SUMMARY) {
            my $text = ( split( /:/, $d ) )[-1];

         #$tree->add( $d, -text => $text, -image => $tree->Getimage("folder") );
            $tree->add( $d, -text => $text );

            #$treetag = $mw->Text();
            #$treetag->insert('end',$d);
            #$tree->add( $treetag, -text => $text);
        }

        # Detect selection changes
        $tree->configure( -command => sub { &summarySelection($textArea) } );

        # Get the + or - functionality
        $tree->autosetmode();

        # Tree is fully expanded by default
        # Just collapse the forsaken thing. We don't want to scare anyone
        foreach (@SUMMARY) {
            $tree->close($_);
        }

        # we got something so we can now save
        # Make sure we get the right label according to the view
        &enableSummaryLabel;

        # If we go to this point we can say we did our first run
        $FIRSTCALL = $ON;
    }    # else $CALLTYPE == $SUMMARYTYPE && Exists($summaryFrame

    #Who call us. Thus the message
    if ( $CALLTYPE == $SCANTYPE ) {
        &display_message("SCANDONE");
    }
    if ( $CALLTYPE == $SUMMARYTYPE ) {
        &display_message("SUMMARYVIEW");
    }
    if ( $CALLTYPE == $DISPLAYTYPE ) {
        &display_message("DISPLAYDONE");
    }

    &restoreCursor;

    # Set the current view
    $CURRENTVIEW = $SUMMARYVIEW;

}

###########################################################
# Routine called when a selection is made in summary report
###########################################################
sub summarySelection {

    # Local variables
    my $colonct = 0;
    my $selection;
    my $prob;
    my $func;
    my $file;
    my $fileline;
    my $linenumber;
    my $filename;
    my $record;
    my $junk;
    my $docfileopen;
    my $startRetrieve;
    my $endRetrieve;
    my $textRetrieve;
    my $editable;

    ($selection) = $tree->selectionGet;

    if ( $DEBUG == $ON ) {
        &log("smsl: In summarySelection subroutine......");
    }

    #find out what we're dealing with
    foreach ( $selection =~ /:/g ) {
        $colonct = $colonct + 1;
    }

    if ( Exists($textArea) ) {
        $textArea->destroy();
    }

    # Get the individual values
    ( $prob, $func, $file, $fileline ) = split( /:/, $selection );

    if ( $colonct == 0 ) {

        # We got a top node
        ( $filename, $junk ) = split( /\</, $prob );
        $filename = $MODULEDIR
          . "/$DOCFILESDIR/"
          . $PORTMPREFIX
          . $filename
          . $DOCEXT
          . $GZEXT;
        $editable = 0;
    }

    if ( $colonct == 1 ) {

    # We got a problem (second node not a real problem for us)
    # Make sure we use the hash value not the function name since some doc files
    # do not correspond to the function name
        ( $func, $junk ) = split( /\</, $func );

        # Get rid of blanks
        $func = $DOCHASH{$func};
        $func =~ s/^\s+//;
        $func =~ s/\s+$//;
        $filename =
          $MODULEDIR . "/$DOCFILESDIR/" . $PORTMPREFIX . $func . $GZEXT;
        $editable = 0;
    }

    if ( $colonct == 2 || $colonct == 3 ) {

        # We got either a file or a filenumber choice
        $filename = $portManager::curr_dir . '/' . $file;
        $editable = 1;
    }

    # Give work impression
    &workCursor;
    &display_message("PLEASEWAIT");
    $mw->update();

    # Look for the file
    if ( !( -e "$filename" ) ) {
        $insert1 = $filename;
        if ( $colonct == 0 || $colonct == 1 ) {
            &display_message("NOTFOUNDINSTALLFILE");
            &log(
"vsrp: Could not find $filename. Check that permission exists to read the file or that file exists."
            );
        }
        else {
            &display_message("FILENOTFOUND");
            &log("vsrp: Could not find $filename. Check that file exists.");
        }
        &restoreCursor;

        # If we got a portout file something is very wrong
        # Directory is out of sync.
        if ( $colonct > 1 ) {
            &display_message("OUTOFSYNCH");
            &log(
"vsrp: Could not find $filename. Base directory may be out of sync."
            );
            &display_message("BLANK");
        }
        return;
    }

    # If the filename exists then load it
    # However, if it doesn't exist we don't make a big stink about it
    # Yet let them know we couldn't load it
    $docfileopen = open( DOCFILE, $filename );

    # Check to see if we have to decompress the read
    #  if ($docfileopen == 1 && ($colonct == 0 || $colonct == 1)) {
    #     open (DOCFILE,"gzip -d -c $filename |");
    #  }

    if ( $docfileopen == 1 ) {
        $textArea = $summaryFrame->Scrolled( qw/TextUndo -scrollbars osoe/,
            -wrap => 'word' );
        $textArea->pack( -side => 'right', -fill => 'both', -expand => 1 );
        $textArea->tagConfigure(
            'high',
            foreground => 'black',
            background => 'red'
        );
        $textArea->tagConfigure(
            'medium',
            foreground => 'black',
            background => 'yellow'
        );
        $textArea->tagConfigure(
            'low',
            foreground => 'black',
            background => 'green'
        );
        $textArea->tagConfigure(
            'unknown',
            foreground => 'black',
            background => 'white'
        );
        $textArea->tagConfigure(
            'unknownmark',
            foreground => 'black',
            background => 'white'
        );
        $textArea->tagConfigure( 'doctag', foreground => 'red' );
        my $lineNumb = 0;
        my $index1   = 0;
        my $index2   = 0;
        my $beginIndex1;
        my $beginIndex2;
        my $endIndex1;
        my $endIndex2;
        my $docIndicator = 0;
        my $tempDoc;
        my $firstString;
        my $lastString;
        my $stringLength;

        while (<DOCFILE>) {

            ####################################################################################
# Check each line for eye catchers for marking.
# Take into account that markings may be more than one or different on the same line
# I'm assuming pair within return delimeter.
            ####################################################################################
            $lineNumb = $lineNumb + 1;

            # Put the line
            $textArea->insert( "end", $_ );

            # Check for difficult tags
            if ( $_ =~ /$structs::DIFFICULTFLAGFRONT/ ) {
                while ( $_ =~ m/$structs::DIFFICULTFLAGFRONT/g ) {
                    $index1 = pos $_;
                    $_ =~ m/$structs::DIFFICULTFLAGBACK/g;
                    $index2 = ( pos $_ ) - 2;
                    $textArea->tagAdd( 'high', "$lineNumb.$index1",
                        "$lineNumb.$index2" );

          # delete tags also make sure you keep track of the current edited line
                    $beginIndex1 = $index1 - 2;
                    $beginIndex2 = $index1;
                    $endIndex1   = $index2;
                    $endIndex2   = $index2 + 2;
                    $textArea->delete( "$lineNumb.$endIndex1",
                        "$lineNumb.$endIndex2" );
                    $textArea->delete( "$lineNumb.$beginIndex1",
                        "$lineNumb.$beginIndex2" );
                    $_ = $textArea->get( "$lineNumb.0", "end" );
                    $textArea->tagBind(
                        'high',
                        "<Double-Button-1>",
                        sub {
                            $startRetrieve =
                              $textArea->index('insert wordstart');
                            $endRetrieve  = $textArea->index('insert wordend');
                            $textRetrieve =
                              $textArea->get( $startRetrieve, $endRetrieve );
                            &getTextDoc($textRetrieve);

                        }
                    );

                    $textArea->tagBind( 'high', "<Any-Enter>",
                        sub { shift->configure( -cursor => 'hand2' ) } );
                    $textArea->tagBind( 'high', "<Any-Leave>",
                        sub { shift->configure( -cursor => 'xterm' ) } );
                }
            }

            # Check for moderate flags
            if ( $_ =~ /$structs::MEDIUMFLAGFRONT/ ) {
                while ( $_ =~ m/$structs::MEDIUMFLAGFRONT/g ) {
                    $index1 = pos $_;
                    $_ =~ m/$structs::MEDIUMFLAGBACK/g;
                    $index2 = ( pos $_ ) - 2;
                    $textArea->tagAdd( 'medium', "$lineNumb.$index1",
                        "$lineNumb.$index2" );

                    # delete tags
                    $beginIndex1 = $index1 - 2;
                    $beginIndex2 = $index1;
                    $endIndex1   = $index2;
                    $endIndex2   = $index2 + 2;
                    $textArea->delete( "$lineNumb.$endIndex1",
                        "$lineNumb.$endIndex2" );
                    $textArea->delete( "$lineNumb.$beginIndex1",
                        "$lineNumb.$beginIndex2" );
                    $_ = $textArea->get( "$lineNumb.0", "end" );
                    $textArea->tagBind(
                        'medium',
                        "<Double-Button-1>",
                        sub {
                            $startRetrieve =
                              $textArea->index('insert wordstart');
                            $endRetrieve  = $textArea->index('insert wordend');
                            $textRetrieve =
                              $textArea->get( $startRetrieve, $endRetrieve );
                            &getTextDoc($textRetrieve);

                        }
                    );

                    $textArea->tagBind( 'medium', "<Any-Enter>",
                        sub { shift->configure( -cursor => 'hand2' ) } );
                    $textArea->tagBind( 'medium', "<Any-Leave>",
                        sub { shift->configure( -cursor => 'xterm' ) } );
                }
            }

            # Check for easy flags
            if ( $_ =~ /$structs::EASYFLAGFRONT/ ) {
                while ( $_ =~ m/$structs::EASYFLAGFRONT/g ) {
                    $index1 = pos $_;
                    $_ =~ m/$structs::EASYFLAGBACK/g;
                    $index2 = ( pos $_ ) - 2;
                    $textArea->tagAdd( 'low', "$lineNumb.$index1",
                        "$lineNumb.$index2" );

                    # delete tags
                    $beginIndex1 = $index1 - 2;
                    $beginIndex2 = $index1;
                    $endIndex1   = $index2;
                    $endIndex2   = $index2 + 2;
                    $textArea->delete( "$lineNumb.$endIndex1",
                        "$lineNumb.$endIndex2" );
                    $textArea->delete( "$lineNumb.$beginIndex1",
                        "$lineNumb.$beginIndex2" );
                    $_ = $textArea->get( "$lineNumb.0", "end" );
                    $textArea->tagBind(
                        'low',
                        "<Double-Button-1>",
                        sub {
                            $startRetrieve =
                              $textArea->index('insert wordstart');
                            $endRetrieve  = $textArea->index('insert wordend');
                            $textRetrieve =
                              $textArea->get( $startRetrieve, $endRetrieve );
                            &getTextDoc($textRetrieve);

                        }
                    );

                    $textArea->tagBind( 'low', "<Any-Enter>",
                        sub { shift->configure( -cursor => 'hand2' ) } );
                    $textArea->tagBind( 'low', "<Any-Leave>",
                        sub { shift->configure( -cursor => 'xterm' ) } );
                }
            }

            # Check for unknown flags
            if ( $_ =~ /$structs::UNKNOWNFLAGFRONT/ ) {
                while ( $_ =~ m/$structs::UNKNOWNFLAGFRONT/g ) {
                    $index1 = pos $_;
                    $_ =~ m/$structs::UNKNOWNFLAGBACK/g;
                    $index2 = ( pos $_ ) - 2;
                    $textArea->tagAdd( 'unknown', "$lineNumb.$index1",
                        "$lineNumb.$index2" );

                    # delete tags
                    $beginIndex1 = $index1 - 2;
                    $beginIndex2 = $index1;
                    $endIndex1   = $index2;
                    $endIndex2   = $index2 + 2;
                    $textArea->delete( "$lineNumb.$endIndex1",
                        "$lineNumb.$endIndex2" );
                    $textArea->delete( "$lineNumb.$beginIndex1",
                        "$lineNumb.$beginIndex2" );
                    $_             = $textArea->get( "$lineNumb.0", "end" );
                    $startRetrieve = $textArea->index('insert wordstart');
                    $endRetrieve   = $textArea->index('insert wordend');
                    $textRetrieve  =
                      $textArea->get( $startRetrieve, $endRetrieve );

                    if ( $DOCHASH{$textRetrieve} ne "" ) {

                        # We clip the tags so we have to account for that
                        $index1 = $index1 - 2;
                        $index2 = $index2 - 2;
                        $textArea->tagAdd( 'unknownmark', "$lineNumb.$index1",
                            "$lineNumb.$index2" );
                        $textArea->tagBind(
                            'unknownmark',
                            "<Double-Button-1>",
                            sub {
                                $startRetrieve =
                                  $textArea->index('insert wordstart');
                                $endRetrieve =
                                  $textArea->index('insert wordend');
                                $textRetrieve =
                                  $textArea->get( $startRetrieve,
                                    $endRetrieve );
                                &getTextDoc($textRetrieve);
                            }
                        );
                        $textArea->tagBind( 'unknownmark', "<Any-Enter>",
                            sub { shift->configure( -cursor => 'hand2' ) } );
                        $textArea->tagBind( 'unknownmark', "<Any-Leave>",
                            sub { shift->configure( -cursor => 'xterm' ) } );
                    }
                }
            }
            if ($docIndicator) {
                $tempDoc = $_;

  # find the first character. Then add the length to first index to find the end
  # of the line. We need this to get the indexes
  # also make sure this is not a blank line with a */ otherwise TK error
  # may occur
                if ( $tempDoc ne $ENDLINE ) {
                    $textArea->tagAdd( 'doctag', "$lineNumb.0",
                        "$lineNumb." . ( length($tempDoc) - 1 ) );
                }

                if ( $_ =~ m/$PORTMANAGERENDDOC/g ) {

                    # Get out of the loop and delete the end delimeters
                    $docIndicator = 0;
                    $index1       = ( pos $_ ) - 2;
                    $index2       = $index1 + 2;
                    $textArea->delete( "$lineNumb.$index1",
                        "$lineNumb.$index2" );
                }
            }

            if ( $_ =~ /$PORTMANAGERDOC/ ) {

                # Find the end of line character and delete to one less
                # Don't delete the whole line otherwise this will not work
                $_ =~ m/$ENDLINE/g;
                $index2 = ( pos $_ ) - 1;
                $textArea->delete( "$lineNumb.0", "$lineNumb.$index2" );
                $docIndicator = 1;
            }

        }    #while

        if ( $colonct == 3 ) {

            # We got a line node Move there. Remember lineindex starts at 0
            ( $junk,       $linenumber ) = split( /\-/, $fileline );
            ( $linenumber, $junk )       = split( /\</, $linenumber );
            $textArea->yview( $linenumber - 1 );
        }

        if ( $colonct == 2 ) {

            # We got a line node Move there. Remember lineindex starts at 0
            my ($firstchild) = $tree->info( 'children', $selection );
            ( undef, undef, undef, $linenumber, undef ) =
              split( /:/, $firstchild );
            $linenumber =~ s/Line-(\d+)<.*/$1/;
            $textArea->yview( $linenumber - 1 );
        }

        $insert1 = $filename;
        &display_message("FILELOADED");
    }
    else {
        if ( $colonct == 0 || $colonct == 1 ) {

            # We got a doc file message is different
            $insert1 = $filename;
            $insert2 = $curr_dir . $LOGFILENAME;
            &log(
"smls: Could not open $filename. Check that file exists or permission exists to read the file."
            );
            &display_message("CRITICALOPENERROR");
            &restoreCursor;
            &display_message("BLANK");
        }
        else {

       # We got a portout file. If this fails we may have an out of sync problem
            $insert1 = $filename;
            $insert2 = $curr_dir . $LOGFILENAME;
            &display_message("CRITICALOPENERROR");
            &log(
"smsl: Could not open $filename. Check that file exists or permission exists to read the file."
            );
            &restoreCursor;
            &display_message("OUTOFSYNCH");
            &log(
"vsrp: Could not find $filename. Base directory may be out of sync."
            );
            &display_message("BLANK");
        }
    }

    close(DOCFILE);

    # Return the previous cursor
    &restoreCursor;
    if ( $editable == 0 ) {
        $textArea->configure( -state => 'disable' );
    } else {
        $textArea->ResetUndo();
    }
}

#######################################################################
# Routine to view the metrics file after a scan or a display
#######################################################################
sub viewMetrics {

    my $metricsopen;
    my $m = 0;
    my @firstpass;
    my $record;
    my $scanReturn;

    if ($DEBUG) {
        &log("vimt: In viewMetrics subroutine");
    }

    if ( $FIRSTCALL == $INIT ) {
        if ( $FIRSTCALL == $OFF ) {
            $file_mbutton->entryconfigure(
                $SAVEASBEGINLABEL,
                -state => 'disabled',
                -label => $SAVEASMETRICSLABEL
            );
        }
        return;
    }

    # view check then we have to make sure we have done a scan or
    # display first
    if ( $CALLTYPE == $METRICTYPE ) {
        if ( $FIRSTCALL == $OFF ) {
            &display_message("NODISPLAY");
            &disableMetricsLabel;
            $CURRENTVIEW = $METRICSVIEW;
            return;
        }

        # We don't know we'll succeed but at least we know where we are
        &disableMetricsLabel;
        $CURRENTVIEW = $METRICSVIEW;
    }

    # Give apperance of work
    &workCursor;

    &display_message("PLEASEWAIT");
    $mw->update();

    # Do a disapearing act but don't want to kill it.
    # We want to keep the view intact in case they move and return
    &hideWidgets;

    # Scan called us we gather info again
    if ( $CALLTYPE == $SCANTYPE ) {
        $scanReturn =
          scanner_wrapper( $curr_dir, $INCLUDEDOC, $MODULEDIR, $message_parm );
        if ( ( $message_parm->{msg_parm_num_messages} ) > 0 ) {
            &process_messages($message_parm);
        }
        if ( $scanReturn == 0 ) {
            @PORTOUTS = get_portouts($message_parm);
            if ( ( $message_parm->{msg_parm_num_messages} ) > 0 ) {
                &process_messages($message_parm);
            }
        }
        else {
            $FIRSTCALL = $OFF;

            # Do all the things when we're not displaying anything
            # Make sure we get the right label according to the view
            $FIRSTCALL = $OFF;
            &disableMetricsLabel;
            &display_message("BLANK");
            &restoreCursor;
            return;
        }
    }

    if ( Exists($metricsFrame) ) {

        # This means no new scan or no new display
        # Just redisplay
        $metricsFrame->pack( -fill => 'both', -expand => 1 );
        &display_message("METRICSVIEW");
        &restoreCursor;
        &enableMetricsLabel;
        return;
    }

    # Check to see if we got something to display
    @PORTOUTS = get_portouts($message_parm);
    if ( ( $message_parm->{msg_parm_num_messages} ) > 0 ) {
        &process_messages($message_parm);
    }

    if ( $PORTOUTS[0] eq '' ) {
        if ( $CALLTYPE == $SCANTYPE ) {
            &display_message("NOPROBFOUND");
        }
        else {
            &display_message("NODISPLAY");
        }
        &disableMetricsLabel;
        &restoreCursor;
        $CURRENTVIEW = $METRICSVIEW;
        $FIRSTCALL   = $OFF;
        return;
    }

    $METRICSFILE = $curr_dir . $METRICSFILENAME;
    chomp($METRICSFILE);

    # Look for the file
    if ( !( -e "$METRICSFILE" ) ) {
        $insert1 = $METRICSFILE;
        &display_message("FILENOTFOUND");
        &log("vsrp: Could not find $METRICSFILE. Check that file exists.");
        $FIRSTCALL = $OFF;
        &disableMetricsLabel;
        &restoreCursor;
        if ( $CALLTYPE == $SCANTYPE ) {
            &display_message("NOPROBFOUND");
        }
        else {
            &display_message("NODISPLAY");
        }
        $CURRENTVIEW = $METRICSVIEW;
        return;
    }

    # Read the metrics input file
    $metricsopen = open( METFILE, $METRICSFILE );

    # Check for return code.
    if ( $metricsopen == 1 ) {
        while ( $record = <METFILE> ) {
            $METRICS[$m] = $record;
            $m = $m + 1;
        }    #while $record
    }
    else {
        $insert1 = $METRICSFILE;
        $insert2 = $curr_dir . $LOGFILENAME;
        &display_message("CRITICALOPENERROR");
        &log(
"vimt: Could not open $METRICSFILE. Check that permission exists to read the file."
        );
        $FIRSTCALL = $OFF;
        &disableMetricsLabel;
        &restoreCursor;
        if ( $CALLTYPE == $SCANTYPE ) {
            &display_message("NOPROBFOUND");
        }
        else {
            &display_message("NODISPLAY");
        }
        $CURRENTVIEW = $METRICSVIEW;
        return;
    }

    close(METFILE);

    if ( $m == 0 ) {

        # Something is not right.
        $insert1 = $METRICSFILE;
        $insert2 = $curr_dir;
        &display_message("EMPTYFILE");
        if ( $CALLTYPE == $SCANTYPE ) {
            &display_message("NOPROBFOUND");
        }
        else {
            &display_message("NODISPLAY");
        }
        $FIRSTCALL = $OFF;
        &disableMetricsLabel;
        &restoreCursor;
        $CURRENTVIEW = $METRICSVIEW;
        return;
    }

    $metricsFrame = $mw->Frame();
    $metricsText  = $metricsFrame->Scrolled(qw/Text -scrollbars osoe/);

    foreach (@METRICS) {
        $metricsText->insert( "end", $_ );
    }

    $metricsTitle = $metricsFrame->Label(
        -text       => "Metrics - Base directory scanned: " . $curr_dir,
        -relief     => 'raised',
        -background => 'grey'
      )->pack(
        -side   => 'top',
        -anchor => 'w',
        -fill   => 'x'
      );

    $metricsText->pack( -fill  => 'both', -expand => 1 );
    $metricsFrame->pack( -fill => 'both', -expand => 1 );

    #Who call us. Thus the message
    if ( $CALLTYPE == $SCANTYPE ) {
        &display_message("SCANDONE");
    }

    if ( $CALLTYPE == $METRICTYPE ) {
        &display_message("METRICSVIEW");
    }

    if ( $CALLTYPE == $DISPLAYTYPE ) {
        &display_message("DISPLAYDONE");
    }

    #If we were able to display we say we have succeded on a call
    $FIRSTCALL   = $ON;
    $CURRENTVIEW = $METRICSVIEW;
    &restoreCursor;
    &enableMetricsLabel;

}

#################################################################
# Routine to view the Graphics for the metrics
#################################################################
sub viewGraphics {
    my $metricsopen;
    my $m = 0;
    my @firstpass;
    my $record;
    my $scanReturn;
    my $baseofdirLabel    = 'Base of directory';
    my $scanFilesLabel    = 'Scanned:';
    my $flaggedFilesLabel = 'Flagged:';
    my $highLabel         = 'High';
    my $mediumLabel       = 'Medium';
    my $lowLabel          = 'Low';
    my $unknownLabel      = 'To be assessed';
    my $temp1;
    my $temp2;
    my $easy;
    my $medium;
    my $diff;
    my $unknown;
    my $startx            = 30;
    my $starty            = 30;
    my $endx              = 305;
    my $endy              = 305;
    my $startDegree       = 0;
    my $startpoint        = 0;
    my $radianCoefficient = .01745;
    my $easyRadian;
    my $mediumRadian;
    my $diffRadian;
    my $unknownRadian;
    my $yLabelPos;
    my $xLabelPos;
    my $radiusOffset = 15;
    my $radius;
    my $rectStartx        = $startx + $endx + 50;
    my $rectStarty        = $starty + 50;
    my $rectEndx          = $rectStartx + 20;
    my $rectEndy          = $rectStarty + 20;
    my $rectLabelx        = $rectStartx + 50;
    my $rectEasyLabely    = $rectEndx;
    my $rectDisplacementy = 40;
    my $titleStartx       = $rectStartx + 40;
    my $titleStarty       = $rectEndy - 80;
    my $stampx            = $titleStartx;
    my $stampy            = $rectEndy - 60;
    my $stamp;
    my $time;
    my $day;
    my $month;
    my $date;
    my $mytime;
    my $year;

    if ($DEBUG) {
        &log("vigp: In viewMetrics subroutine");
    }

    if ( $FIRSTCALL == $INIT ) {
        if ( $FIRSTCALL == $OFF ) {
            $file_mbutton->entryconfigure(
                $SAVEASBEGINLABEL,
                -state => 'disabled',
                -label => $SAVEASGRAPHICSLABEL
            );
        }
        return;
    }

    # view check then we have to make sure we have done a scan or
    # display first
    if ( $CALLTYPE == $GRAPHICTYPE ) {
        if ( $FIRSTCALL == $OFF ) {
            &display_message("NODISPLAY");
            &disableGraphicsLabel;
            $CURRENTVIEW = $GRAPHICSVIEW;
            return;
        }

        # We don't know we'll succeed but at least we know where we are
        &disableGraphicsLabel;
        $CURRENTVIEW = $GRAPHICSVIEW;
    }

    # Give apperance of work
    &workCursor;

    &display_message("PLEASEWAIT");
    $mw->update();

    # Do a disapearing act but don't want to kill it.
    # We want to keep the view intact in case they move and return
    &hideWidgets;

    # Scan called us we gather info again
    if ( $CALLTYPE == $SCANTYPE ) {
        $scanReturn =
          scanner_wrapper( $curr_dir, $INCLUDEDOC, $MODULEDIR, $message_parm );
        if ( ( $message_parm->{msg_parm_num_messages} ) > 0 ) {
            &process_messages($message_parm);
        }
        if ( $scanReturn == 0 ) {
            @PORTOUTS = get_portouts($message_parm);
            if ( ( $message_parm->{msg_parm_num_messages} ) > 0 ) {
                &process_messages($message_parm);
            }
        }
        else {
            $FIRSTCALL = $OFF;

            # Do all the things when we're not displaying anything
            # Make sure we get the right label according to the view
            $FIRSTCALL = $OFF;
            &disableGraphicsLabel;
            &display_message("BLANK");
            &restoreCursor;
            return;
        }
    }

    if ( Exists($graphicsFrame) ) {

        # This means no new scan or no new display
        # Just redisplay
        $graphicsFrame->pack( -fill => 'both', -expand => 1 );
        &enableGraphicsLabel;
        &display_message("GRAPHICSVIEW");
        &restoreCursor;
        return;
    }

    # Check to see if we got something to display
    @PORTOUTS = get_portouts($message_parm);
    if ( ( $message_parm->{msg_parm_num_messages} ) > 0 ) {
        &process_messages($message_parm);
    }

    if ( $PORTOUTS[0] eq '' ) {
        if ( $CALLTYPE == $SCANTYPE ) {
            &display_message("NOPROBFOUND");
        }
        else {
            &display_message("NODISPLAY");
        }
        &disableGraphicsLabel;
        &restoreCursor;
        $CURRENTVIEW = $GRAPHICSVIEW;
        $FIRSTCALL   = $OFF;
        return;
    }

    $METRICSFILE = $curr_dir . $METRICSFILENAME;
    chomp($METRICSFILE);

    # Check for file existance
    if ( !( -e "$METRICSFILE" ) ) {
        $insert1 = $METRICSFILE;
        &display_message("FILENOTFOUND");
        &log("vsrp: Could not find $METRICSFILE. Check that file exists.");
        $FIRSTCALL = $OFF;
        &disableGraphicsLabel;
        &restoreCursor;
        if ( $CALLTYPE == $SCANTYPE ) {
            &display_message("NOPROBFOUND");
        }
        else {
            &display_message("NODISPLAY");
        }
        $CURRENTVIEW = $GRAPHICSVIEW;
        return;
    }

    # Read the metrics input file
    $metricsopen = open( METFILE, $METRICSFILE );

    # Check for return code.
    if ( $metricsopen == 1 ) {
        while ( $record = <METFILE> ) {

            # Note that the following code is based on the assumption that the
            # $somethingLABEL variables contain the same text that was written
            # into the metrics file by the routine port_code_scan.pm:scanner.

            $METRICS[$m] = $record;

            # Check and gather percent numbers and other information
            if ( $record =~ /$baseofdirLabel/ ) {
                $METRICSBASEDIR = $record;
            }
            if ( $record =~ /$scanFilesLabel/ ) {
                $METRICSSCANFILES = $record;
            }
            if ( $record =~ /$flaggedFilesLabel/ ) {
                $METRICSFLAGGED = $record;
            }
            if ( $record =~ /$highLabel/ ) {
                ( $temp1, $temp2 )       = split( /%/,    $record );
                ( $temp2, $HIGHPERCENT ) = split( /[()]/, $temp1 );
            }
            if ( $record =~ /$mediumLabel/ ) {
                ( $temp1, $temp2 )         = split( /%/,    $record );
                ( $temp2, $MEDIUMPERCENT ) = split( /[()]/, $temp1 );
            }
            if ( $record =~ /$lowLabel/ ) {
                ( $temp1, $temp2 )      = split( /%/,    $record );
                ( $temp2, $LOWPERCENT ) = split( /[()]/, $temp1 );
            }
            if ( $record =~ /$unknownLabel/ ) {
                ( $temp1, $temp2 )          = split( /%/,    $record );
                ( $temp2, $UNKNOWNPERCENT ) = split( /[()]/, $temp1 );
            }

            $m = $m + 1;
        }    #while $record
    }
    else {
        $insert1 = $METRICSFILE;
        $insert2 = $curr_dir . $LOGFILENAME;
        &display_message("CRITICALOPENERROR");
        &log(
"vigp: Could not open $METRICSFILE. Check that file exists or permission exists to read the file."
        );
        $FIRSTCALL = $OFF;
        &disableGraphicsLabel;
        &restoreCursor;
        if ( $CALLTYPE == $SCANTYPE ) {
            &display_message("NOPROBFOUND");
        }
        else {
            &display_message("NODISPLAY");
        }
        $CURRENTVIEW = $GRAPHICSVIEW;
        return;
    }

    close(METFILE);

    if ( $m == 0 ) {

        # Something is not right.
        $insert1 = $METRICSFILE;
        $insert2 = $curr_dir;
        &display_message("EMPTYFILE");
        if ( $CALLTYPE == $SCANTYPE ) {
            &display_message("NOPROBFOUND");
        }
        else {
            &display_message("NODISPLAY");
        }
        $FIRSTCALL = $OFF;
        &disableGraphicsLabel;
        &restoreCursor;
        $CURRENTVIEW = $GRAPHICSVIEW;
        return;
    }

    $graphicsFrame  = $mw->Frame();
    $graphicsCanvas = $graphicsFrame->Scrolled(qw/Canvas -scrollbars osoe/);

    $graphicsTitle = $graphicsFrame->Label(
        -text       => "Graphics - Base directory scanned: " . $curr_dir,
        -relief     => 'raised',
        -background => 'grey'
      )->pack(
        -side   => 'top',
        -anchor => 'w',
        -fill   => 'x'
      );

    $graphicsCanvas->pack( -side => 'right', -fill => 'both', -expand => 1 );
    $graphicsFrame->pack( -fill => 'both', -expand => 1 );

    $easy    = $LOWPERCENT * 3.6;
    $medium  = $MEDIUMPERCENT * 3.6;
    $diff    = $HIGHPERCENT * 3.6;
    $unknown = $UNKNOWNPERCENT * 3.6;

    # Let's bake us a pie
    if ( $easy > 0 ) {

        # 360 just prints a line
        if ( $easy == 360 ) {
            $easy = 359.99;
            $graphicsCanvas->create(
                ( "arc", $startx, $starty, $endx, $endy ),
                -style  => 'chord',
                -fill   => 'green',
                -start  => $startpoint,
                -extent => $easy
            );
        }
        else {
            $graphicsCanvas->create(
                ( "arc", $startx, $starty, $endx, $endy ),
                -fill   => 'green',
                -start  => $startpoint,
                -extent => $easy
            );
        }
    }

    if ( $medium > 0 ) {

        # 360 just prints a line
        if ( $medium == 360 ) {
            $medium = 359.99;
            $graphicsCanvas->create(
                ( "arc", $startx, $starty, $endx, $endy ),
                -style  => 'chord',
                -fill   => 'yellow',
                -start  => $easy,
                -extent => $medium
            );
        }
        else {
            $graphicsCanvas->create(
                ( "arc", $startx, $starty, $endx, $endy ),
                -fill   => 'yellow',
                -start  => $easy,
                -extent => $medium
            );
        }
    }

    if ( $diff > 0 ) {

        # 360 just prints a line
        if ( $diff == 360 ) {
            $diff = 359.99;
            $graphicsCanvas->create(
                ( "arc", $startx, $starty, $endx, $endy ),
                -style  => 'chord',
                -fill   => 'red',
                -start  => $easy + $medium,
                -extent => $diff
            );
        }
        else {
            $graphicsCanvas->create(
                ( "arc", $startx, $starty, $endx, $endy ),
                -fill   => 'red',
                -start  => $easy + $medium,
                -extent => $diff
            );
        }
    }

    # Given rounding imprecitions we may need to sacrifice someone as
    # opposed to overlapping. Since we don't know these people let's
    # pick them.
    $temp1 = ( $diff + $easy + $medium + $unknown );
    if ( $temp1 != 360 && $unknown > 0 ) {
        $unknown = ( 360 - $diff - $easy - $medium );
    }

    # 360 gives us problems. Nothing gets displayed
    if ( $unknown == 360 ) {
        $unknown = 359.99;
        $graphicsCanvas->create(
            ( "arc", $startx, $starty, $endx, $endy ),
            -style  => 'chord',
            -fill   => 'white',
            -start  => $easy + $medium + $diff,
            -extent => $unknown
        );
    }
    else {
        $graphicsCanvas->create(
            ( "arc", $startx, $starty, $endx, $endy ),
            -fill   => 'white',
            -start  => $easy + $medium + $diff,
            -extent => $unknown
        );
    }

# Now we need the labels. Oh boy what do we do now?
# No el problemo. Nothing a little trigonometry can't solve.
# This is what's going on. Given that the cartesian plane for a
# canvas starts at origen 0,0 on the top left hand side of the window.
# Then our circle starts at origen (0 + $startx + $endx / 2), (0 + $starty + $endy / 2)
# The end result is that we end up with a circle at that position with a radius of
# half the $yend or $xend value minus the $startx and $starty.
# Given this information and the angle. We can project a radius slightly higher than the
# radius of the circle and drive it throuth the middle of the current angle. By using
# cosine and sine functions we can find our new projected x and y coordinates.
# The degrees also have to be converted to radians for this to work.
# The other problem facing us is that if the circle has a displacement, meaning that,
# startx and starty are not 0, we need to compensate for that.

    $easyRadian = $easy * $radianCoefficient;
    $radius     = ( $endx - $startx ) / 2;
    $xLabelPos  =
      $radius + ( ( $radius + $radiusOffset ) * cos( $easyRadian / 2 ) );
    $yLabelPos =
      $radius - ( ( $radius + $radiusOffset ) * sin( $easyRadian / 2 ) );
    $xLabelPos = $xLabelPos + $startx;
    $yLabelPos = $yLabelPos + $starty;

    # No negatives allow in this cartesian plane
    if ( $yLabelPos < 0 ) {
        $yLabelPos = $yLabelPos * -1;
    }
    if ( $xLabelPos < 0 ) {
        $xLabelPos = $xLabelPos * -1;
    }

    # If label is 0 then don't print the label
    if ( $LOWPERCENT > 0 ) {
        $graphicsCanvas->create(
            ( "text", $xLabelPos, $yLabelPos ),
            -text => "$LOWPERCENT" . "%"
        );
    }

    # If we add the first angle and add half of next we got what we want
    $mediumRadian = ( $easy + ( $medium / 2 ) ) * $radianCoefficient;
    $xLabelPos = $radius + ( $radius + $radiusOffset ) * cos($mediumRadian);
    $yLabelPos = $radius - ( $radius + $radiusOffset ) * sin($mediumRadian);
    $xLabelPos = $xLabelPos + $startx;
    $yLabelPos = $yLabelPos + $starty;

    # No negatives allow in this cartesian plane
    if ( $yLabelPos < 0 ) {
        $yLabelPos = $yLabelPos * -1;
    }
    if ( $xLabelPos < 0 ) {
        $xLabelPos = $xLabelPos * -1;
    }

    # If label is 0 then don't print the label
    if ( $MEDIUMPERCENT > 0 ) {
        $graphicsCanvas->create(
            ( "text", $xLabelPos, $yLabelPos ),
            -text => "$MEDIUMPERCENT" . "%"
        );
    }

    $diffRadian = ( $easy + $medium + ( $diff / 2 ) ) * $radianCoefficient;
    $xLabelPos = $radius + ( $radius + $radiusOffset ) * cos($diffRadian);
    $yLabelPos = $radius - ( $radius + $radiusOffset ) * sin($diffRadian);
    $xLabelPos = $xLabelPos + $startx;
    $yLabelPos = $yLabelPos + $starty;

    # No negatives allow in this cartesian plane
    if ( $yLabelPos < 0 ) {
        $yLabelPos = $yLabelPos * -1;
    }
    if ( $xLabelPos < 0 ) {
        $xLabelPos = $xLabelPos * -1;
    }

    # If label is 0 then don't print the label
    if ( $HIGHPERCENT > 0 ) {
        $graphicsCanvas->create(
            ( "text", $xLabelPos, $yLabelPos ),
            -text => "$HIGHPERCENT" . "%"
        );
    }

    $unknownRadian =
      ( $easy + $medium + $diff + ( $unknown / 2 ) ) * $radianCoefficient;
    $xLabelPos = $radius + ( $radius + $radiusOffset ) * cos($unknownRadian);
    $yLabelPos = $radius - ( $radius + $radiusOffset ) * sin($unknownRadian);
    $xLabelPos = $xLabelPos + $startx;
    $yLabelPos = $yLabelPos + $starty;

    # No negatives allow in this cartesian plane
    if ( $yLabelPos < 0 ) {
        $yLabelPos = $yLabelPos * -1;
    }
    if ( $xLabelPos < 0 ) {
        $xLabelPos = $xLabelPos * -1;
    }

    # If label is 0 then don't print the label
    if ( $UNKNOWNPERCENT > 0 ) {
        $graphicsCanvas->create(
            ( "text", $xLabelPos, $yLabelPos ),
            -text => "$UNKNOWNPERCENT" . "%"
        );
    }

    # Let's build the color labels
    $graphicsCanvas->create(
        ( "rectangle", $rectStartx, $rectStarty, $rectEndx, $rectEndy ),
        -fill => 'green' );

    $graphicsCanvas->create(
        (
            "rectangle",                          $rectStartx,
            $rectStarty + $rectDisplacementy * 1, $rectEndx,
            $rectEndy + $rectDisplacementy * 1
        ),
        -fill => 'yellow'
    );

    $graphicsCanvas->create(
        (
            "rectangle",                          $rectStartx,
            $rectStarty + $rectDisplacementy * 2, $rectEndx,
            $rectEndy + $rectDisplacementy * 2
        ),
        -fill => 'red'
    );

    $graphicsCanvas->create(
        (
            "rectangle",                          $rectStartx,
            $rectStarty + $rectDisplacementy * 3, $rectEndx,
            $rectEndy + $rectDisplacementy * 3
        ),
        -fill => 'white'
    );

    # Create title in case they save it
    $graphicsCanvas->create(
        ( "text", $titleStartx, $titleStarty ),
        -text => "SourceCheckingTool Graphics Report  "
    );

    $stamp = time();
    $time  = localtime($stamp);

    ( $day, $month, $date, $mytime, $year ) = split( /\s+/, $time );

    $graphicsCanvas->create( ( "text", $stampx, $stampy ),
        -text => "$day $month $date $year $mytime  " );

    # Create Labels for the rectangles
    $graphicsCanvas->create( ( "text", $rectStartx + 50, $rectEndy - 10 ),
        -text => "Low     " );

    $graphicsCanvas->create(
        (
            "text",
            $rectStartx + 50,
            $rectEndy + ( $rectDisplacementy * 1 - 10 )
        ),
        -text => "Medium "
    );

    $graphicsCanvas->create(
        (
            "text",
            $rectStartx + 50,
            $rectEndy + ( $rectDisplacementy * 2 - 10 )
        ),
        -text => "High    "
    );

    $graphicsCanvas->create(
        (
            "text",
            $rectStartx + 74,
            $rectEndy + ( $rectDisplacementy * 3 - 10 )
        ),
        -text => "To be assessed"
    );

    # Place other labels
    $graphicsCanvas->create(
        ( "text", $endx / 2, $endy + 35 ),
        -text   => "$METRICSBASEDIR",
        -anchor => 'w'
    );

    $graphicsCanvas->create(
        ( "text", $endx / 2, $endy + 55 ),
        -text   => "$METRICSSCANFILES",
        -anchor => 'w'
    );

    $graphicsCanvas->create(
        ( "text", $endx / 2, $endy + 70 ),
        -text   => "$METRICSFLAGGED",
        -anchor => 'w'
    );

    $graphicsCanvas->update;

    #Who call us. Thus the message
    if ( $CALLTYPE == $SCANTYPE ) {
        &display_message("SCANDONE");
    }

    if ( $CALLTYPE == $METRICTYPE ) {
        &display_message("METRICSVIEW");
    }

    if ( $CALLTYPE == $DISPLAYTYPE ) {
        &display_message("DISPLAYDONE");
    }

    #If we were able to display we say we have succeded on a call
    $FIRSTCALL   = $ON;
    $CURRENTVIEW = $GRAPHICSVIEW;
    &restoreCursor;
    &display_message("GRAPHICSVIEW");
    &enableGraphicsLabel;

}

#######################################
# Routine to change to working cursor
#######################################
sub workCursor {
    $OLDCURSOR = $mw->cget('-cursor');
    $mw->configure( -cursor => 'watch' );
}

#######################################
# Routine to change to old cursor
#######################################
sub restoreCursor {
    $mw->configure( -cursor => $OLDCURSOR );
}

###################################################################
# Routine to overwrite some menu items for TextUndo file menu items
###################################################################
sub Tk::TextUndo::FileMenuItems {
    my ($w) = @_;
    return [

        # Some menu items like Exit and Save do not make sense
        # according to our spec and content.
        # Exit doesn't make sense because it blows the application up
        # Save doesn't make sense because we want to save the user
        # from overwritting an output file. Save As makes sense.
        [ "command" => '~Open...',    -command => [ $w => 'FileLoadPopup' ] ],
        [ "command" => 'Save ~As...', -command => [ $w => 'FileSaveAsPopup' ] ],
        [
            "command" => '~Include...',
            -command => [ $w => 'IncludeFilePopup' ]
        ],
        [ "command" => '~Clear', -command => [ $w => 'ConfirmEmptyDocument' ] ]
    ];
}

#####################################################################
# Routine to save the main window in to a file.
# It overloads the original Tk routine.
#####################################################################

sub Tk::TextUndo::Save
{
 my ($w,$filename) = @_;
 $filename = $w->FileName unless defined $filename;
 return $w->FileSaveAsPopup unless defined $filename;
 my $layers = $w->PerlIO_layers;
 if (open(my $file,">$layers",$filename))
  {
   my $index = '1.0';
   my ($lines) = $w->index('end - 1 chars') =~ /^(\d+)\./;
   while ($w->compare($index,'<','end'))
    {
     my $end = $w->index("$index  lineend +1c");
     print $file $w->get($index,$end);
     $index = $end;
    }
   if (close($file))
    {
     $w->ResetUndo;
     $w->FileName($filename);
     return 1;
    }
  }
 else
  {
   $insert1 = "window content";
   $insert2 = $filename;
   &display_message("NOPERMWRITEERROR");
   &log("svas: Could not save window content in $filename. Check for permissions to save the file.");
  }
 return 0;
}


############################################################$########
# Routine to overwrite some menu items for TextUndo search menu items
# Actually these values are really under Text
#####################################################################
sub Tk::TextUndo::SearchMenuItems {
    my ($w) = @_;
    return [
        [ 'command' => '~Find...', -command => [ $w => 'FindPopUp' ] ],
        [
            'command' => 'Find ~Next',
            -command => [ $w => 'FindSelectionNext' ]
        ],
        [
            'command' => 'Find ~Previous',
            -command => [ $w => 'FindSelectionPrevious' ]
        ],
        [
            'command' => '~Replace...',
            -command => [ $w => 'FindAndReplacePopUp' ]
        ]
    ];
}

############################################################$########
# Routine to overwrite some menu items for Text on the help menu items
#####################################################################
sub Tk::Text::SearchMenuItems {
    my ($w) = @_;
    return [
        [ 'command' => '~Find...', -command => [ $w => 'FindPopUp' ] ],
        [
            'command' => 'Find ~Next',
            -command => [ $w => 'FindSelectionNext' ]
        ],
        [
            'command' => 'Find ~Previous',
            -command => [ $w => 'FindSelectionPrevious' ]
        ]
    ];
}

#######################################
# Routine to log messages in debug mode
#######################################
sub log {

    #print "In log";
    if ( !( -e "$curr_dir/SCT_files" ) ) {
        if ( !( mkdir "$curr_dir/SCT_files" ) ) {
            $insert1 = $curr_dir;
            &display_message("ROOTSCANDIRPROB");
            return;
        }
    }
    my $stamp = time();
    my $time  = localtime($stamp);
    $LOGFILE = $curr_dir . $LOGFILENAME;

    # Check that we are able to open the file to append. Also only display
    # the pop up message once.
    if ( !open( LOG, ">> $LOGFILE" ) ) {
        if ( $logReturn == 0 ) {
            $insert1 = "any messages";
            $insert2 = "$LOGFILE.";
            &display_message("WRITEERROR");
            $logReturn = 1;
            return $logReturn;
        }
    }
    else {
        print LOG "$time |@_\n";
        close(LOG);
        $logReturn = 0;
        return $logReturn;
    }
}

#############################################################
# Routine to track all Perl internal Errors into the log file
#############################################################
sub Tk::Error {
    my $messageWindow;
    my $message;
    my $choice;

    # if regular cursor has been initialized then restore it
    &restoreCursor;
    print @_, "\n";    # Give some background indication something is wrong
    &log("TkEr: @_");  # Always log to log file

    # Something is not right so make a big fuzz
    $insert1 = $curr_dir . $LOGFILENAME;
    &display_message("TKERROR");
    &display_message("BLANK");

}

##############################################################
# Routine to display the table of content help
##############################################################

# subroutine used locally by function "help" to handle the event handler
sub getContentHelp {
    my $file;
    my $label;

    # Check for the type of help we want
    if ( $helpSelection eq $guiLabel ) {
        $label = $guiLabel;
        $file  = $guiFile;
    }

    if ( $helpSelection eq $viewLabel ) {
        $label = $viewLabel;
        $file  = $viewFile;
    }

    if ( $helpSelection eq $editorLabel ) {
        $label = $editorLabel;
        $file  = $editorFile;
    }

    if ( $helpSelection eq $messageLabel ) {
        $label = $messageLabel;
        $file  = $messageFile;
    }

    if ( $helpSelection eq $buildLabel ) {
        $label = $buildLabel;
        $file  = $buildFile;
    }

    # Initialize the help window
    if ( Exists($helpPanel) ) {
        $helpPanel->destroy;
    }
    $helpPanel = $mw->Toplevel;
    $helpPanel->title( "Help - " . $label );
    $helpFrame = $helpPanel->Frame()->pack( -fill => 'both', -expand => 1 );
    $helpText = $helpFrame->Scrolled(
        'Text',
        -width      => 100,
        -setgrid    => 'no',
        -scrollbars => 'osoe',
        -relief     => 'groove',
        -wrap       => 'word'
    );

    # Check for file existance
    if ( !( -e "$MODULEDIR" . "$file" ) ) {
        $insert1 = $MODULEDIR . $file;
        &display_message("NOTFOUNDINSTALLFILE");
        &log(   "vsrp: Could not find $MODULEDIR"
              . "$file. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root."
        );
        $helpPanel->destroy;
        return;
    }

    $openFile = open( TEXTFILE, $MODULEDIR . $file );
    if ( $openFile == 1 ) {

        # We're rolling
        while (<TEXTFILE>) {
            $helpText->insert( "end", $_ );
        }
    }
    else {

        # End of the road
        $insert1 = $MODULEDIR . $file;
        $insert2 = $curr_dir . $LOGFILENAME;
        &display_message("CRITICALOPENERROR");
        &log(   "help: Could not open $MODULEDIR"
              . "$file. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root."
        );
        $helpPanel->destroy;
        return;
    }

    # A little layout
    $helpText->pack( -side => 'top', -expand => 'yes', -fill => 'both' );
    $helpText->configure( -state => 'disable' );

    $helpFrame->Button(
        -text    => '  Back  ',
        -command => sub { $ok = 1 }
      )->pack(
        -side => 'left',
        -padx => 140,
      );
    $helpFrame->Button(
        -text    => '   Close   ',
        -command => sub { $ok = -1 }
      )->pack(
        -side => 'left',
        -padx => 10,
      );

    #Wait for some input
    $helpFrame->waitVariable( \$ok );

    # We need this again. We can't call help again because we end up with a
    # recursion problem if we try to find a short that way
    if ( $ok == 1 ) {

        # initialize the window
        if ( Exists($helpPanel) ) {
            $helpPanel->destroy;
        }
        $helpPanel = $mw->Toplevel;
        $helpPanel->title("Help - Table of Contents");
        $helpFrame = $helpPanel->Frame()->pack( -fill => 'both', -expand => 1 );
        $helpList = $helpFrame->ScrlListbox(
            -width      => 40,
            -setgrid    => 'no',
            -selectmode => 'single',
            -scrollbars => 'osoe',
            -relief     => 'groove'
        );

        $helpFrame->Button(
            -text    => '  Close  ',
            -command => sub { $helpPanel->destroy }
        )->pack( -side => 'bottom', -padx => 40 );

        # Try to find and read the content file
        if ( !( -e "$MODULEDIR" . "$contentFile" ) ) {
            $insert1 = $MODULEDIR . $contentFile;
            &display_message("NOTFOUNDINSTALLFILE");
            &log(   "vsrp: Could not find $MODULEDIR"
                  . "$contentFile. ScoureCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root."
            );
            $helpPanel->destroy;
            return;
        }

        # Try to read and content help
        $openFile = open( CONTENTFILE, $MODULEDIR . $contentFile );

        if ( $openFile == 1 ) {
            while (<CONTENTFILE>) {

                # Replace line endings with spaces
                $_ =~ s/(\r\n?|\n)//g;
                $helpList->insert( "end", $_ );
            }
        }
        else {
            $insert1 = $MODULEDIR . $contentFile;
            $insert2 = $curr_dir . $LOGFILENAME;
            &display_message("CRITICALOPENERROR");
            &log(   "help: Could not open $MODULEDIR"
                  . "$contentFile. SourceCheckingTol may not have been installed correctly. Try reinstalling SourceCheckingTool as root.."
            );
            $helpPanel->destroy;
            return;
        }

        $helpList->bind(
            "<Button-1>",
            sub {
                ( $helpSelection =
                      ( $helpList->get( $helpList->curselection() ) ) ),
                  &getContentHelp,;
            }
        );

        $helpList->pack( -side => 'top', -expand => 'yes', -fill => 'both' );
    }    # ok=1

    # So you want to get out?
    if ( $ok == -1 ) {
        if ( Exists($helpPanel) ) {
            $helpPanel->destroy;
        }
        return;
    }
}    # end getContentHelp

#
#
# Main help routine.
sub help {
    if ( $DEBUG == $ON ) {
        &log("help: In help subroutine......");
    }

    # Do basic clean up
    if ( Exists($helpPanel) ) {
        $helpPanel->destroy;
    }

    # Set the main window
    $helpPanel = $mw->Toplevel;
    $helpPanel->title("Help - Table of Contents");
    $helpFrame = $helpPanel->Frame()->pack( -fill => 'both', -expand => 1 );
    $helpList = $helpFrame->ScrlListbox(
        -width      => 40,
        -setgrid    => 'no',
        -selectmode => 'single',
        -scrollbars => 'osoe',
        -relief     => 'groove'
    );

    # Try to find and read the content file
    if ( !( -e "$MODULEDIR" . "$contentFile" ) ) {
        $insert1 = "$MODULEDIR" . $contentFile;
        &display_message("NOTFOUNDINSTALLFILE");
        &log(   "vsrp: Could not find $MODULEDIR"
              . "$contentFile. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root."
        );
        $helpPanel->destroy;
        return;
    }

    $openFile = open( CONTENTFILE, $MODULEDIR . $contentFile );
    if ( $openFile == 1 ) {

        # We're in business
        while (<CONTENTFILE>) {

            # Replace line endings with spaces
            $_ =~ s/(\r\n?|\n)//g;
            $helpList->insert( "end", $_ );
        }
    }
    else {

        # Oh shoot
        $insert1 = "$MODULEDIR" . $contentFile;
        $insert2 = $curr_dir . $LOGFILENAME;
        &display_message("CRITICALOPENERROR");
        &log(   "help: Could not open $MODULEDIR"
              . "$contentFile. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root."
        );
        $helpPanel->destroy;
        return;
    }

    # Set the event handler
    $helpList->bind(
        "<Button-1>",
        sub {
            ( $helpSelection =
                  ( $helpList->get( $helpList->curselection() ) ) ),
              &getContentHelp,;
        }
    );

    $helpList->pack( -side => 'top', -expand => 'yes', -fill => 'both' );

    $helpFrame->Button(
        -text    => '  Close  ',
        -command => sub { $helpPanel->destroy }
    )->pack( -side => 'bottom', -padx => 40 );

}    # end help()

##############################################################
# Routine to display the About Help
##############################################################
sub about {
    my $openFile;

    if ( $DEBUG == $ON ) {
        &log("abot: In About subroutine......");
    }

    # Do basic clean up
    if ( Exists($helpPanel) ) {
        $helpPanel->destroy;
    }

    # Set the window
    $helpPanel = $mw->Toplevel;
    $helpPanel->title("Help - About");
    $helpFrame = $helpPanel->Frame()->pack( -fill => 'both', -expand => 1 );
    $helpText = $helpFrame->Scrolled(
        'Text',
        -width      => 100,
        -setgrid    => 'no',
        -scrollbars => 'osoe',
        -relief     => 'groove',
        -wrap       => 'word'
    );

    # Try to find and read the content file
    if ( !( -e "$MODULEDIR" . "$aboutFile" ) ) {
        $insert1 = $MODULEDIR . $aboutFile;
        &display_message("NOTFOUNDINSTALLFILE");
        &log(   "vsrp: Could not find $MODULEDIR"
              . "$aboutFile. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root."
        );
        $helpPanel->destroy;
        return;
    }

    $openFile = open( TEXTFILE, $MODULEDIR . $aboutFile );
    if ( $openFile == 1 ) {

        #We're rolling
        while (<TEXTFILE>) {
            $helpText->insert( "end", $_ );
        }
    }
    else {

        # End of the road
        $insert1 = $MODULEDIR . $aboutFile;
        $insert2 = $curr_dir . $LOGFILENAME;
        &display_message("CRITICALOPENERROR");
        &log(   "abot: Could not open $MODULEDIR"
              . "$aboutFile. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root."
        );
        $helpPanel->destroy;
        return;
    }

    # A little layout
    $helpText->pack( -side => 'top', -expand => 'yes', -fill => 'both' );
    $helpText->configure( -state => 'disable' );

    $helpFrame->Button(
        -text    => '  Close  ',
        -command => sub { $ok = 1 }
      )->pack(
        -side => 'bottom',
        -padx => 40,
      );

    #Wait for some input
    $helpFrame->waitVariable( \$ok );

    # No brainer
    if ( $ok == 1 ) {
        $helpPanel->destroy;
    }

}    # end about()

###########################################################
# Routine to enable summary label
###########################################################
sub enableSummaryLabel {

    if ($DEBUG) {
        &log("eslb: In enableSummaryLabel subroutine");
    }
    if ( $CURRENTVIEW == $SUMMARYVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASSUMMARYLABEL,
            -state => 'normal',
            -label => $SAVEASSUMMARYLABEL
        );
    }
    if ( $CURRENTVIEW == $METRICSVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASMETRICSLABEL,
            -state => 'normal',
            -label => $SAVEASSUMMARYLABEL
        );
    }
    if ( $CURRENTVIEW == $PORTOUTVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASPORTOUTLABEL,
            -state => 'normal',
            -label => $SAVEASSUMMARYLABEL
        );
    }
    if ( $CURRENTVIEW == $GRAPHICSVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASGRAPHICSLABEL,
            -state => 'normal',
            -label => $SAVEASSUMMARYLABEL
        );
    }
}

###########################################################
# Routine to disable summary label
###########################################################
sub disableSummaryLabel {

    if ($DEBUG) {
        &log("dslb: In disableSummaryLabel subroutine");
    }
    if ( $CURRENTVIEW == $SUMMARYVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASSUMMARYLABEL,
            -state => 'disabled',
            -label => $SAVEASSUMMARYLABEL
        );
    }
    if ( $CURRENTVIEW == $METRICSVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASMETRICSLABEL,
            -state => 'disabled',
            -label => $SAVEASSUMMARYLABEL
        );
    }
    if ( $CURRENTVIEW == $PORTOUTVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASPORTOUTLABEL,
            -state => 'disabled',
            -label => $SAVEASSUMMARYLABEL
        );
    }
    if ( $CURRENTVIEW == $GRAPHICSVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASGRAPHICSLABEL,
            -state => 'disabled',
            -label => $SAVEASSUMMARYLABEL
        );
    }
}

###########################################################
# Routine to enable portout label
###########################################################
sub enablePortoutLabel {

    if ($DEBUG) {
        &log("eplb: In enablePortoutLabel subroutine");
    }
    if ( $CURRENTVIEW == $SUMMARYVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASSUMMARYLABEL,
            -state => 'normal',
            -label => $SAVEASPORTOUTLABEL
        );
    }
    if ( $CURRENTVIEW == $METRICSVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASMETRICSLABEL,
            -state => 'normal',
            -label => $SAVEASPORTOUTLABEL
        );
    }
    if ( $CURRENTVIEW == $PORTOUTVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASPORTOUTLABEL,
            -state => 'normal',
            -label => $SAVEASPORTOUTLABEL
        );
    }
    if ( $CURRENTVIEW == $GRAPHICSVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASGRAPHICSLABEL,
            -state => 'normal',
            -label => $SAVEASPORTOUTLABEL
        );
    }
}

###########################################################
# Routine to disable portout label
###########################################################
sub disablePortoutLabel {

    if ($DEBUG) {
        &log("dplb: In disablePortoutLabel subroutine");
    }
    if ( $CURRENTVIEW == $SUMMARYVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASSUMMARYLABEL,
            -state => 'disabled',
            -label => $SAVEASPORTOUTLABEL
        );
    }
    if ( $CURRENTVIEW == $METRICSVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASMETRICSLABEL,
            -state => 'disabled',
            -label => $SAVEASPORTOUTLABEL
        );
    }
    if ( $CURRENTVIEW == $PORTOUTVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASPORTOUTLABEL,
            -state => 'disabled',
            -label => $SAVEASPORTOUTLABEL
        );
    }
    if ( $CURRENTVIEW == $GRAPHICSVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASGRAPHICSLABEL,
            -state => 'disabled',
            -label => $SAVEASPORTOUTLABEL
        );
    }
}

###########################################################
# Routine to enable metrics label
###########################################################
sub enableMetricsLabel {

    if ($DEBUG) {
        &log("emlb: In enableMetricsLabel subroutine");
    }
    if ( $CURRENTVIEW == $SUMMARYVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASSUMMARYLABEL,
            -state => 'normal',
            -label => $SAVEASMETRICSLABEL
        );
    }
    if ( $CURRENTVIEW == $METRICSVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASMETRICSLABEL,
            -state => 'normal',
            -label => $SAVEASMETRICSLABEL
        );
    }
    if ( $CURRENTVIEW == $PORTOUTVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASPORTOUTLABEL,
            -state => 'normal',
            -label => $SAVEASMETRICSLABEL
        );
    }
    if ( $CURRENTVIEW == $GRAPHICSVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASGRAPHICSLABEL,
            -state => 'normal',
            -label => $SAVEASMETRICSLABEL
        );
    }
}

###########################################################
# Routine to disable metrics label
###########################################################
sub disableMetricsLabel {

    if ($DEBUG) {
        &log("dmlb: In disableMetricsLabel subroutine");
    }
    if ( $CURRENTVIEW == $SUMMARYVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASSUMMARYLABEL,
            -state => 'disabled',
            -label => $SAVEASMETRICSLABEL
        );
    }
    if ( $CURRENTVIEW == $METRICSVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASMETRICSLABEL,
            -state => 'disabled',
            -label => $SAVEASMETRICSLABEL
        );
    }
    if ( $CURRENTVIEW == $PORTOUTVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASPORTOUTLABEL,
            -state => 'disabled',
            -label => $SAVEASMETRICSLABEL
        );
    }
    if ( $CURRENTVIEW == $GRAPHICSVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASGRAPHICSLABEL,
            -state => 'disabled',
            -label => $SAVEASMETRICSLABEL
        );
    }
}

###########################################################
# Routine to enable graphics label
###########################################################
sub enableGraphicsLabel {

    if ($DEBUG) {
        &log("dglb: In enableGraphicsLabel subroutine");
    }
    if ( $CURRENTVIEW == $SUMMARYVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASSUMMARYLABEL,
            -state => 'normal',
            -label => $SAVEASGRAPHICSLABEL
        );
    }
    if ( $CURRENTVIEW == $METRICSVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASMETRICSLABEL,
            -state => 'normal',
            -label => $SAVEASGRAPHICSLABEL
        );
    }
    if ( $CURRENTVIEW == $PORTOUTVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASPORTOUTLABEL,
            -state => 'normal',
            -label => $SAVEASGRAPHICSLABEL
        );
    }
    if ( $CURRENTVIEW == $GRAPHICSVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASGRAPHICSLABEL,
            -state => 'normal',
            -label => $SAVEASGRAPHICSLABEL
        );
    }
}

###########################################################
# Routine to disable graphics label
###########################################################
sub disableGraphicsLabel {

    if ($DEBUG) {
        &log("dglb: In disableGraphicsLabel subroutine");
    }
    if ( $CURRENTVIEW == $SUMMARYVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASSUMMARYLABEL,
            -state => 'disabled',
            -label => $SAVEASGRAPHICSLABEL
        );
    }
    if ( $CURRENTVIEW == $METRICSVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASMETRICSLABEL,
            -state => 'disabled',
            -label => $SAVEASGRAPHICSLABEL
        );
    }
    if ( $CURRENTVIEW == $PORTOUTVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASPORTOUTLABEL,
            -state => 'disabled',
            -label => $SAVEASGRAPHICSLABEL
        );
    }
    if ( $CURRENTVIEW == $GRAPHICSVIEW ) {
        $file_mbutton->entryconfigure(
            $SAVEASGRAPHICSLABEL,
            -state => 'disabled',
            -label => $SAVEASGRAPHICSLABEL
        );
    }
}

###########################################################
# Routine to display the documentation for a clicked tag
###########################################################
sub getTextDoc {

    my @chosenTag = @_;
    my $hashname;
    my $filename;
    my $docfileopen;
    my $record;
    my @docText;
    my $m = 0;
    my $docWindow;
    my $docFrame;
    my $docText;

    if ($DEBUG) {
        &log("gtxd: In getTextDoc subroutine");
    }

    if (!defined($DOCHASH{ $chosenTag[0]} )) {
        $insert1 = $chosenTag[0];
        &display_message( "FUNCTIONEDITED" );
        return;
    }

    $hashname = $DOCHASH{ $chosenTag[0] };
    chomp($hashname);

    # Build the file name
    $filename =
      $MODULEDIR . "/$DOCFILESDIR/" . $PORTMPREFIX . $hashname . $GZEXT;

    # Look for the file
    if ( !( -e "$filename" ) ) {
        $insert1 = $filename;
        &display_message("NOTFOUNDINSTALLFILE");
        &log(
"vsrp: Could not find $filename. Check that file exists or permission exist to read directory."
        );
        &restoreCursor;
        return;
    }

    # If the filename exists then load it
    # However, if it doesn't exist we don't make a big stink about it
    # Yet let them know we couldn't load it
    $docfileopen = open( DOCFILE, $filename );

    # This is a zip file
    if ( $docfileopen == 1 ) {
        open( DOCFILE, $filename );

        while ( $record = <DOCFILE> ) {
            $docText[$m] = $record;
            $m = $m + 1;
        }

        close(DOCFILE);

        # Uhmmm... Somthing is not right
        if ( $m == 0 ) {

            # Something is not right.
            $insert1 = $filename;
            $insert2 = $curr_dir;
            &display_message("EMPTYFILE");
            if ( $CALLTYPE == $SCANTYPE ) {
                &display_message("NOPROBFOUND");
            }
            else {
                &display_message("NODISPLAY");
            }
            return;
        }

        # Let's build the output window
        $docWindow = $mw->Toplevel();
        $docWindow->title("$chosenTag[0]");
        $docFrame = $docWindow->Frame()->pack( -fill => 'both', -expand => 1 );
        $docText = $docFrame->Scrolled(
            'Text',
            -wrap       => 'word',
            -setgrid    => 'no',
            -scrollbars => 'osoe',
            -relief     => 'groove'
        );

        # Place info
        foreach (@docText) {
            $docText->insert( "end", $_ );
        }

        $docText->configure( -state => 'disable' );

        $docText->pack( -side => 'top', -expand => 'yes', -fill => 'both' );

        $docFrame->Button(
            -text    => '  Close  ',
            -command => sub { $docWindow->destroy; },
        )->pack();

    }
    else {
        $insert1 = $filename;
        $insert2 = $curr_dir . $LOGFILENAME;
        &log(
"smls: Could not open $filename. Check that file exists or permission exists to read the file."
        );
        &display_message("CRITICALOPENERROR");
        &restoreCursor;
        &display_message("BLANK");
    }
}

##############################################################
# Check for main files. If any of these are missing. We got
# most likely an installation problem.
##############################################################
sub initCheckVitals {
    my $name;     #BP
    my $value;    #BP
    my $vitalReturn = 0;
    my $openfile;

  PATH: foreach (@INC) {
        $openfile = open( SCANPM, "$_" . "$PORTMINSTBASEDIR" . $scanMod );
        if ($openfile) {
            $MODULEDIR = $_ . $PORTMINSTBASEDIR;
            close(SCANPM);
            last PATH;
        }
    }

    # did we find something?
    if ( $MODULEDIR eq '' ) {
        print
"Unable to find the installation path. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root.\n";
        $vitalReturn = 1;
        return $vitalReturn;
    }

    # Check for read access in directories
    if ( !( -r "$MODULEDIR" . "$portInsDir" ) ) {
        print "Unable to read $MODULEDIR"
          . "$portInsDir. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root.\n";
        $vitalReturn = 1;
        return $vitalReturn;
    }

    if ( !( -r "$MODULEDIR" . "$portDocDir" ) ) {
        print "Unable to read $MODULEDIR"
          . "$portDocDir. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root.\n";
        $vitalReturn = 1;
        return $vitalReturn;
    }

    if ( !( -r "$MODULEDIR" . "$portHelpDir" ) ) {
        print "Unable to read $MODULEDIR"
          . "$portHelpDir. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root.\n";
        $vitalReturn = 1;
        return $vitalReturn;
    }

    # If we got this far. Let's check for existance.
    if ( !( -e $MODULEDIR . $messageTable ) ) {
        print "Unable to find $MODULEDIR"
          . "$messageTable. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root.\n";
        $vitalReturn = 1;
        return $vitalReturn;
    }

    if ( !( -e $MODULEDIR . $guiFile ) ) {
        print "Unable to find $MODULEDIR"
          . "$guiFile. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root.\n";
        $vitalReturn = 1;
        return $vitalReturn;
    }

    if ( !( -e $MODULEDIR . $buildFile ) ) {
        print "Unable to find $MODULEDIR"
          . "$buildFile. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root.\n";
        $vitalReturn = 1;
        return $vitalReturn;
    }

    if ( !( -e $MODULEDIR . $aboutFile ) ) {
        print "Unable to find $MODULEDIR"
          . "$aboutFile. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root.\n";
        $vitalReturn = 1;
        return $vitalReturn;
    }

    if ( !( -e $MODULEDIR . $funcs ) ) {
        print "Unable to find $MODULEDIR"
          . "$funcs. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root. \n";
        $vitalReturn = 1;
        return $vitalReturn;
    }

    if ( !( -e $MODULEDIR . $incls ) ) {
        print "Unable to find $MODULEDIR"
          . "$incls. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root.\n";
        $vitalReturn = 1;
        return $vitalReturn;
    }

    if ( !( -e $MODULEDIR . $parameters ) ) {
        print "Unable to find $MODULEDIR"
          . "$parameters. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root.\n";
        $vitalReturn = 1;
        return $vitalReturn;
    }

    if ( !( -e $MODULEDIR . $pragmas ) ) {
        print "Unable to find $MODULEDIR"
          . "$pragmas. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root.\n";
        $vitalReturn = 1;
        return $vitalReturn;
    }

    # if unable to open message table put out print and return BP
    # If we're not able to open here we have an installation problem.
    if ( !open( MSGFILE, $MODULEDIR . $messageTable ) ) {
        print "Unable to open $MODULEDIR"
          . "$messageTable. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root.\n";
        $vitalReturn = 1;
        return $vitalReturn;
    }

    # BP
    while (<MSGFILE>) {
        ( $name, $value ) = split( "#", $_ );
        $messages{$name} = $value;
        $msg_index++;
    }    # while $msg_record

    close(MSGFILE);

    if ( $msg_index == 0 ) {

        # Something is not right.  Can't call message component if this fails.
        # if the message table is empty, it's probably an install problem
        print "Message table file: $MSGFILE is empty\n";
        $vitalReturn = 1;
        return $vitalReturn;
    }

    if ( !open( GUIFILE, $MODULEDIR . $guiFile ) ) {
        print "Unable to open $MODULEDIR"
          . "$guiFile. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root.\n";
        $vitalReturn = 1;
        return $vitalReturn;
    }

    close(GUIFILE);

    if ( !open( BUILDFILE, $MODULEDIR . $buildFile ) ) {
        print "Unable to open $MODULEDIR"
          . "$buildFile. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root.\n";
        $vitalReturn = 1;
        return $vitalReturn;
    }

    close(BUILDFILE);

    if ( !open( ABOUTFILE, $MODULEDIR . $aboutFile ) ) {
        print "Unable to open $MODULEDIR"
          . "$aboutFile. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root.\n";
        $vitalReturn = 1;
        return $vitalReturn;
    }

    close(ABOUTFILE);

    if ( !open( FUNCSFILE, $MODULEDIR . $funcs ) ) {
        print "Unable to open $MODULEDIR"
          . "$funcs. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root. \n";
        $vitalReturn = 1;
        return $vitalReturn;
    }

    close(FUNCSFILE);

    if ( !open( INCLSFILE, $MODULEDIR . $incls ) ) {
        print "Unable to open $MODULEDIR"
          . "$incls. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root.\n";
        $vitalReturn = 1;
        return $vitalReturn;
    }

    close(INCLSFILE);

    if ( !open( PARAMETERSFILE, $MODULEDIR . $parameters ) ) {
        print "Unable to open $MODULEDIR"
          . "$parameters. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root.\n";
        $vitalReturn = 1;
        return $vitalReturn;
    }

    close(PARAMETERSFILE);

    if ( !open( PRAGMASFILE, $MODULEDIR . $pragmas ) ) {
        print "Unable to open $MODULEDIR"
          . "$pragmas. SourceCheckingTool may not have been installed correctly. Try reinstalling SourceCheckingTool as root.\n";
        $vitalReturn = 1;
        return $vitalReturn;
    }

    close(PRAGMASFILE);

    return $vitalReturn;
}

##############################################################
# process_message - takes the message_parm parameter list and
#                   process a message list
#                   &display_message is called with the hash
#                   value passed in.
#                   insert1 and insert2 are setup prior to the call
##############################################################
sub process_messages (\$) {
    my ($message_parm) = @_;

#
# Process message array returned from the scanner
# if num_messages is non-zero, enter a loop, as the
#    scanner may return up to 4 messages.
#    The message_info_array_struct struct contains:
#      message_info_array_hashkey, message_info_array_insert1, message_info_array_insert2
#      pull out the key and inserts, issue the message
# Reset num_messages to 0
#
    $message_row = 0;    # initialize message row number
    for (
        my $loop_cntl = 0 ;
        $loop_cntl < ( $message_parm->{msg_parm_num_messages} ) ;
        $loop_cntl++
      )
    {
        $insert1 = $message_parm->{msg_info_array_insert1}[$message_row];
        $insert2 = $message_parm->{msg_info_array_insert2}[$message_row];
        &display_message(
            $message_parm->{msg_info_array_hashkey}[$message_row] );
        $message_row = $message_row + 1;    # increment row number
    }
    $message_parm->{msg_parm_num_messages} =
      0;                                    # reset number of messages to zero.
}
