#!/bin/bash
#
# Linux Compatablity Tool Endian errors in global variable declarations checker
#  
#  Module: chkendian_globals.bash
#
#  Purpose: parse the objdump.txt file for info about global variables
#
#  Usage:
#        chkendian_globals.bash
#
#  Inputs:
#    $DB environment variable
#    $EXEFILE environment variable
#
#  Outputs:
#    tGlobalVarDef.tbl
#
#  Notes:
#    EXENAME is the image basename
#    EXEFILE is the full image name and path
#   

if [[ -z $DB ]]
then
    echo "Missing assignment"
    echo "    DB environment variable not set"
    exit 1
fi

if [[ -z $EXEFILE ]]
then
    echo "Missing assignment"
    echo "    EXEFILE environment variable not set"
    exit 1
fi

if [[ -z $ECTBIN ]]
then
    echo "Missing assignment"
    echo "    ECTBIN environment variable not set"
    exit 1
fi

if [[ -z $OBJDUMP ]]
then
    echo "Missing assignment"
    echo "    OBJDUMP environment variable not set"
    exit 1
fi

ggrep " \.bss" $OBJDUMP  |ggrep -v "object\.2"  | 
			      ggrep -v " stdout$"    |
			      ggrep -v " stderr$"    |
                              ggrep -v " stdout@@GLIBC_2.0$" |
                              ggrep -v " stderr@@GLIBC_2.0$" |
                              ggrep -v "@@GLIBC_2.0$" |
			gawk '{  flg=0;
			varname = $NF
                        if ( $2 != "g" )        flg=1;
                        if ( $2 == ".bss" )     flg=1;
                        if ( $3 == "DO" )       flg=1;
                        if ( $5 == "00000000" ) flg=1;
                        if ( $6 == "" )         flg=1;
                        sub("[.][0-9]+$","", varname)
                        if ( flg == 0 ) 
                        print $1" " $5" " varname }' > $DB/objdump.tmp
                           

ggrep " \.data" $OBJDUMP  | gawk '{ flg=0;
			varname = $NF
                        if ( $2 == ".data" )       flg=1;
                        if (substr($0,0,1) == " ") flg=1;
                        if ( $5 == "00000000" )    flg=1;
                        if ( $4 == "00000000" )    flg=1;
                        if ( $1 == "Contents" )    flg=1;
                        if ( $1 == "Disassembly" ) flg=1;
                        sub("[.][0-9]+$","", varname)
                        if ( flg == 0 ) print $1" " $5" "  varname " \""$0"\""}' >> $DB/objdump.tmp

[ $ECTSTAT ] && echo "	ECT 0% - `date`" >> $ECTSTAT

gegrep -v " vtable | typeinfo " $DB/objdump.tmp > $DB/objdump.1

${ECTBIN}/get_global_var_def_info.exe  $DB/ $EXEFILE

[ $ECTSTAT ] && echo "	ECT 50% - `date`" >> $ECTSTAT

#${ECTBIN}/get_global_var_ref_info.bash
${ECTBIN}/getGlobalVarRefInfo.exe 

[ $ECTSTAT ] && echo "	ECT 100% - `date`" >> $ECTSTAT

#rm -f $DB/objdump.1 $DB/objdump.2 $DB/objdump.3

