#!/bin/bash

if [[ -z ${DB} ]]
then
    echo "Missing assignment"
    echo "    DB environment variable not set"
    exit 1
fi

if [[ -z ${EXEFILE} ]]
then
    echo "Missing assignment"
    echo "    EXEFILE environment variable not set"
    exit 1
fi

if [[ -z ${ECTBIN} ]]
then
    echo "Missing assignment"
    echo "    ECTBIN environment variable not set"
    exit 1
fi

if [ -f ${DB}/addr2line.out ]
then
   rm  ${DB}/addr2line.out
fi

if [ -f ${DB}/gdbcommands.txt ]
then
   rm ${DB}/gdbcommands.txt
fi

if [ -f ${DB}/gdbcommands2.txt ]
then
   rm ${DB}/gdbcommands2.txt
fi

if [ -f ${DB}/gdbcommands3.txt ]
then
   rm ${DB}/gdbcommands3.txt
fi

if [ -f ${DB}/info_functions.gdbout ]
then
   rm ${DB}/info_functions.gdbout
fi

if [ -f ${DB}/tFunctionDef.tbl ]
then
   rm ${DB}/tFunctionDef.tbl
fi

if [ -f ${DB}/tFunctionDefParms.tbl ]
then
   rm ${DB}/tFunctionDefParms.tbl
fi

if [ -f ${DB}/print_sizeof.gdbout ]
then
   rm ${DB}/print_sizeof.gdbout
fi

[ $ECTSTAT ] && echo "	ECT 0% - `date`" >> $ECTSTAT

#####################################################################
# The following creates the gdb_info_lines.cmd file for the gdb
#####################################################################
$ECTBIN/crtgdb.exe
[ $ECTSTAT ] && echo "	ECT 25% - `date`" >> $ECTSTAT
#####################################################################
# Run gdb in batch mode with the command file from above
#####################################################################
> $DB//raw_info_functions.gdbout
cp $DB/gdb_info_lines.cmd $DB/gdb_info_lines.cmd.save
todo=1
while [ $todo -eq 1 ]
do
	todo=0
	gdb --readnow --batch --command=$DB/gdb_info_lines.cmd $EXEFILE >> $DB/raw_info_functions.gdbout 2> $DB/gdb_info_lines.error

	errorString=`cat $DB/gdb_info_lines.error`
	if [ ! -z "$errorString" ]
	then
		cp $DB/gdb_info_lines.error $DB/gdb_info_lines.error.last
		todo=1
		errorLine=`echo $errorString | gawk -F ":" '{print $2}'`
		#echo "errorLine = $errorLine"

		nextline=$(($errorLine+1))
		#echo "nextline = $nextline"

		
		# this should print the error line so we know what it is
		#sed -n "$errorLine,$errorLine"p  $DB/gdb_info_lines.cmd

		mv $DB/gdb_info_lines.cmd $DB/gdb_info_lines.cmd.last
		echo "set width 0" > $DB/gdb_info_lines.cmd
		echo "define 1"   >> $DB/gdb_info_lines.cmd
		echo ""           >> $DB/gdb_info_lines.cmd
		echo "end"        >> $DB/gdb_info_lines.cmd
		sed -n "$nextline,$"p  $DB/gdb_info_lines.cmd.last >> $DB/gdb_info_lines.cmd
	fi
done


[ $ECTSTAT ] && echo "	ECT 50% - `date`" >> $ECTSTAT
#####################################################################
# Trim the data before giving it to the next step.
# Then take the $DB/info_functions.gdbout and use
# use gdb to get the details about the sizes and types of each argument
# for the functions defined. This script creates two files 
# $DB/tFunctionDef.tbl and $DB/tFunctionDefParams.tbl
#####################################################################
$ECTBIN/crtFuncDef.exe $DB/raw_info_functions.gdbout $DB/info_functions.gdbout

#gawk -f ${ECTBIN}/funcdef.awk ${DB}/info_functions.gdbout 
#gawk -f /tmp/funcdef.awk ${DB}/info_functions.gdbout 
exit 0

[ $ECTSTAT ] && echo "	ECT 100% - `date`" >> $ECTSTAT

exit 0
