#!/usr/bin/perl -Tw

$numArgs = $#ARGV + 1;
if( $numArgs < 1)
{
	printf( " Error:\n\t usage <tune.pl <inputfile>\n");
	exit;
}

$print_type=0;
$data_file=$ARGV[0];
if( $numArgs == 2)
{
	$print_type=$ARGV[1];
}

open ( INFILE, $data_file) || die ( "could not open file $data_file"); 
@raw_data=<INFILE>;
close (INFILE);

my @words;

my %E30001data;
my %E30001otherdata;

my %E30002data;
my %E30002otherdata;

foreach $line ( @raw_data)
{
	if ( $line =~ /E30001/) {
		update( $line, \%E30001data, \%E30001otherdata);
	}
	elsif ( $line =~ /E30002/) {
		update( $line, \%E30002data, \%E30002otherdata);
	}
}

if( $print_type == 1) {
	print_concise_E30001();
	print "\n\n";
	print_concise_E30002();
}elsif( $print_type == 2) {
	print_combo();
}else {
	print_full_E30001();
	print "\n\n";
	print_full_E30002();
}

exit;

sub print_full_E30001 {
	printf( "========================================================\n");
	printf( "=============   E30001     =============================\n");
	my $count=0;
	foreach my $key ( sort keys %E30001data) {
		printf( "========================================================\n");
		my @values = split( / /, $E30001data{ $key});
		printf( "FUNCTION=\"%s\"  LINE=%s  FILE=%s\n", $key, $values[1], $values[0]);
		my @data = @{ $E30001otherdata{$key}};
		for ( my $i=0; $i < @data;) {
			printf( "  arg=%-2s size=(%s:%s) line=%-5s %s\n", $data[$i], $data[$i+1], $data[$i+4], $data[$i+2], $data[$i+3]);
			$i=$i+5;
			$count++;
		}
	}
	my $dlen = keys %E30001data;
	printf( "\nTOTAL FUNCTIONS = %s    TOTAL FUNCATION CALLS = %s   AVERAGE CALL PER FUNCTION = %s\n", $dlen, $count, int( $count/$dlen));
}

sub print_full_E30002 {
	printf( "========================================================\n");
	printf( "=============   E30002     =============================\n");
	my $count=0;
	foreach my $key ( sort keys %E30002data) {
		printf( "========================================================\n");
		my @values = split( / /, $E30002data{ $key});
		printf( "FUNCTION=\"%s\"  LINE=%s  FILE=%s\n", $key, $values[1], $values[0]);
		my @data = @{ $E30002otherdata{$key}};
		for ( my $i=0; $i < @data;) {
			printf( "  arg=%-2s type=(%s:%s) line=%-5s %s\n", $data[$i], $data[$i+1], $data[$i+4], $data[$i+2], $data[$i+3]);
			$i=$i+5;
			$count++;
		}
	}
	my $dlen = keys %E30002data;
	printf( "\nTOTAL FUNCTIONS = %s    TOTAL FUNCATION CALLS = %s   AVERAGE CALL PER FUNCTION = %s\n", $dlen, $count, int( $count/$dlen));
}

sub update
{
	my $line = shift( @_);
	my $alldata_ref = shift( @_);
	my $otherdata_ref = shift( @_);

	my @words = split ( / /, $line);

	my ($ifunc, $ifile, $iline, $it1, $it2 );
	if ( $words[10] eq "in"){
		($ifunc, $ifile, $iline, $it1, $it2 ) = ( 13, 16, 19, 21, 22);
	}elsif ( $words[11] eq "in"){
		($ifunc, $ifile, $iline, $it1, $it2 ) = ( 14, 17, 20, 22, 23);
	}elsif ( $words[12] eq "in"){
		($ifunc, $ifile, $iline, $it1, $it2 ) = ( 15, 18, 21, 23, 24);
	}else{
		($ifunc, $ifile, $iline, $it1, $it2 ) = ( 16, 19, 22, 24, 25);
	}

	my ($function ) = split( /,/, $words[$ifunc]);
	if( !defined( $alldata_ref->{$function}) )
	{
		$alldata_ref->{$function} = $words[$ifile]." ".$words[$iline];
	}
	push( @ { $otherdata_ref->{ $function } }, $words[7]);
	if( $words[10] eq "in"){
		push( @ { $otherdata_ref->{ $function } }, $words[9]);
	} elsif( $words[11] eq "in"){
		push( @ { $otherdata_ref->{ $function } }, ($words[9]." ".$words[10]));
	} elsif( $words[12] eq "in"){
		push( @ { $otherdata_ref->{ $function } }, ($words[9]." ".$words[10]." ".$words[11]));
	}else {
		push( @ { $otherdata_ref->{ $function } }, ($words[9]." ".$words[10]." ".$words[11]." ".$words[12]));
	}
	my ($cline, @dummy) = split( /\t/, $words[3]);
	push( @ { $otherdata_ref->{ $function } }, $cline);
	my ($cline1, $cline2, @dummy1);
	$cline2 = $words[0];
	if( $words[0] =~  /:/) {
		($cline1, $cline2, @dummy1) = split( /:/, $words[0]);
	}
	push( @ { $otherdata_ref->{ $function } }, $cline2);
	if( defined( $words[$it2])) {
		my ($cline3, @dummy2) = split( /\)/, $words[$it2]);
		my $cline4 = $words[$it1]." ". $cline3;
		push( @ { $otherdata_ref->{ $function } }, $cline4);
	}
	else {
		my ($cline5, @dummy2) = split( /\)/, $words[$it1]);
		push( @ { $otherdata_ref->{ $function } }, $cline5);
	}
}

sub print_concise_E30001 {
	printf( "E30001:\n");
	my $count=0;
	foreach my $key ( sort keys %E30001data) {
		my @values = split( / /, $E30001data{ $key});
		printf( "  FUNCTION=\"%s\"  LINE=%s  FILE=%s\n", $key, $values[1], $values[0]);
		my @data = @{ $E30001otherdata{$key}};
		my %tmplist;
		for ( my $i=0; $i < @data;) {
			if( !defined( $tmplist{ $data[$i]})){
				printf( "    arg=%-2s size=(%s:%s) line=%-5s %s\n", $data[$i], $data[$i+1], $data[$i+4], $data[$i+2], $data[$i+3]);
				$tmplist{ $data[$i]}=1;
			}
			$i=$i+5;
		}
		$count++;
	}
	printf( "\nUNIQUE E30001 = $count\n");
}

sub print_concise_E30002 {
	printf( "E30002:\n");
	my $count=0;
	foreach my $key ( sort keys %E30002data) {
		my @values = split( / /, $E30002data{ $key});
		printf( "  FUNCTION=\"%s\"  LINE=%s  FILE=%s\n", $key, $values[1], $values[0]);
		my @data = @{ $E30002otherdata{$key}};
		my %tmplist;
		for ( my $i=0; $i < @data;) {
			if( !defined( $tmplist{ $data[$i]})){
				printf( "    arg=%-2s type=(%s:%s) line=%-5s %s\n", $data[$i], $data[$i+1], $data[$i+4], $data[$i+2], $data[$i+3]);
				$tmplist{ $data[$i]}=1;
			}
			$i=$i+5;
		}
		$count++;
	}
	printf( "\nUNIQUE E30002 = $count\n");
}


sub print_combo {
	my $count=0;
	foreach my $key ( sort keys %E30001data) {
		if( !defined( $E30002data{$key})) {
			next;
		}
		printf( "--------------------------------------------------------\n");
		my @values = split( / /, $E30001data{ $key});
		printf( "FUNCTION=\"%s\"  LINE=%s  FILE=%s\n", $key, $values[1], $values[0]);
		printf( "  E30001:\n");
		my @data = @{ $E30001otherdata{$key}};
		my %tmplist;
		for ( my $i=0; $i < @data;) {
			if( !defined( $tmplist{ $data[$i]})){
				printf( "    arg=%-2s size=(%s:%s) line=%-5s %s\n", $data[$i], $data[$i+1], $data[$i+4], $data[$i+2], $data[$i+3]);
				$tmplist{ $data[$i]}=1;
			}
			$i=$i+5;
		}

		@values = split( / /, $E30002data{ $key});
		printf( "  E30002:\n");
		@data = @{ $E30002otherdata{$key}};
		my %tmplist2;
		for ( my $i=0; $i < @data;) {
			if( !defined( $tmplist2{ $data[$i]})){
				printf( "    arg=%-2s type=(%s:%s) line=%-5s %s\n", $data[$i], $data[$i+1], $data[$i+4], $data[$i+2], $data[$i+3]);
				$tmplist2{ $data[$i]}=1;
			}
			$i=$i+5;
		}
		$count++;
		delete $E30001data{$key};
		delete $E30002data{$key};
	}
	printf( "\nUNIQUE E30001/E30002 COMBINATIONS = $count\n");
	printf( "--------------------------------------------------------\n");
	print_concise_E30001();
	printf( "--------------------------------------------------------\n");
	print_concise_E30002();
}

